# ***********************************************************************************
# ************************* VUNIT AND LIBRARY INITIALIZATION ************************
# ***********************************************************************************

# Load required libraries from VUnit
from os.path import join, dirname
# The public interface of VUnit
from vunit import VUnit
# Computes the cartesian product of input iterables
from itertools import product
# Load required functions for testcase files load
from os import listdir

# Returns the directory name where the present file (run.py) is located
root = dirname(__file__)

# Create VUnit instance by parsing command line arguments
ui = VUnit.from_argv()

# Add random numbers generation package
ui.add_random()
# Add verification component library
ui.add_verification_components()
# Add communication package
ui.add_com()

# Create library 'vunit_lib'
vunit_lib = ui.library("vunit_lib")

# Add all package files
vunit_lib.add_source_files(join(root, "../../../Image/_packages/*.vhd"))
vunit_lib.add_source_files(join(root, "../../../Predictor/_packages/*.vhd"))
vunit_lib.add_source_files(join(root, "../../../Encoder/_packages/*.vhd"))

# Add all sources files from Encoder header
vunit_lib.add_source_files(join(root, "../*.vhd"))

# Add testbench file from Encoder header IP
vunit_lib.add_source_files(join(root, "tb_top_enc_header.vhd"))

# ***********************************************************************************
# ************************************ FUNCTIONS ************************************
# ***********************************************************************************

# To encode the parameters, the script must contain the encode function
def encode(tb_cfg):
    return ",".join(["%s:%s" % (key, str(tb_cfg[key])) for key in tb_cfg])

# A list of parameters are defined, then saved and finally encoded to be in the testbench
def gen_enc_header_tests(obj, smpl_type, smpl_order, predict_mode, lsum_type, w_init_type, per_err_lim_upd, fidel_type, abs_err_band, rel_err_band, encoder_type, supl_tbl_type, supl_tbl_struct):
    for smpl_type, smpl_order, predict_mode, lsum_type, w_init_type, per_err_lim_upd, fidel_type, abs_err_band, rel_err_band, encoder_type, supl_tbl_type, supl_tbl_struct in product(smpl_type, smpl_order, predict_mode, lsum_type, w_init_type, per_err_lim_upd, fidel_type, abs_err_band, rel_err_band, encoder_type, supl_tbl_type, supl_tbl_struct):
        tb_cfg = dict(
            SMPL_TYPE_PY=smpl_type,
            SMPL_ORDER_PY=smpl_order,
            PREDICT_MODE_PY=predict_mode,
            LSUM_TYPE_PY=lsum_type,
            W_INIT_TYPE_PY=w_init_type,
            PER_ERR_LIM_UPD_PY=per_err_lim_upd,
            FIDEL_CTRL_TYPE_PY=fidel_type,
            ABS_ERR_BAND_TYPE_PY=abs_err_band,
            REL_ERR_BAND_TYPE_PY=rel_err_band,
            ENCODER_TYPE_PY=encoder_type,
            SUPL_TABLE_TYPE_PY=supl_tbl_type,
            SUPL_TABLE_STRUCT_PY=supl_tbl_struct
        )
        config_name = encode(tb_cfg)
        obj.add_config(name=config_name, generics=dict(encoded_tb_cfg=encode(tb_cfg)))

# ***********************************************************************************
# ****************************** GENERATE TESTBENCHES *******************************
# ***********************************************************************************

# Everytime a new testbench is here requested (it can be the same with different parameters),
# all test cases in the VHDL testbench file will be executed again.
tb_top_enc_header = vunit_lib.test_bench("tb_top_enc_header")
for test in tb_top_enc_header.get_tests():
    # gen_enc_header_tests(test, ['0', '1'], [0, 1, 2], ['0', '1'], [0, 1, 2, 3], ['0', '1'], ['0', '1'], [0, 1, 2, 3], ['0', '1'], ['0', '1'], [0, 1], [0, 1, 2], [0, 1, 2, 3])
    gen_enc_header_tests(test, ['0'], [1, 2], ['1'], [0], ['0'], ['0', '1'], [3], ['1'], ['1'], [0, 1], [1], [1])

# ***********************************************************************************
# ********************************** MAIN FUNCTION **********************************
# ***********************************************************************************

ui.main()