library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- Generic labels (constants) to be used anywhere in the source code, so that it is more readable
package gen_labels is

	-----------------------------------------------------------------------------------------------------------
	--  Time (in clock cycles) that every (sub-)block needs to produce a valid output, givin a valid input
	constant TIME_ADDER_C		: integer := 1;
	constant TIME_QUANTIZER_C	: integer := 2;
	constant TIME_SMPL_REPRE_C	: integer := 3;
	constant TIME_PREDICTION_C	: integer := 8;
	constant TIME_MAPPER_C		: integer := 2;
	-- For the predictor to produce a vaild output data, closed loop elements are not necessary here
	constant TIME_PREDICTOR_C	: integer := TIME_ADDER_C + TIME_QUANTIZER_C + TIME_MAPPER_C;
	
	constant TIME_ENC_HEADER_C	: integer := 0;
	constant TIME_SMPL_ADAPT_C	: integer := 3;
	constant TIME_HYBRID_C		: integer := 4;
	constant TIME_ENC_BODY_C	: integer := TIME_HYBRID_C;		-- Simply the entropy coder type that has the longest processing time
	constant TIME_PACKER_C		: integer := 1;
	constant TIME_ENCODER_C		: integer := TIME_ENC_HEADER_C + TIME_ENC_BODY_C + TIME_PACKER_C;
	
	constant TIME_INTERCONN_C	: integer := 1;
	constant TIME_CCSDS123_C	: integer := TIME_PREDICTOR_C + TIME_INTERCONN_C + TIME_ENCODER_C;
	
	-----------------------------------------------------------------------------------------------------------
	-- Values for "SMPL_ORDER_G"
	constant BSQ_C				: std_logic_vector(1 downto 0) := "00";
	constant BIP_C				: std_logic_vector(1 downto 0) := "01";
	constant BIL_C				: std_logic_vector(1 downto 0) := "10";
	
	-----------------------------------------------------------------------------------------------------------
	-- Values for "LSUM_TYPE_G"
	constant WID_NEI_C			: std_logic_vector(1 downto 0) := "00";
	constant NAR_NEI_C			: std_logic_vector(1 downto 0) := "01";
	constant WID_COL_C			: std_logic_vector(1 downto 0) := "10";
	constant NAR_COL_C			: std_logic_vector(1 downto 0) := "11";
	
	-----------------------------------------------------------------------------------------------------------
	-- Values for "FIDEL_CTRL_TYPE_G"
	constant LOSSLESS_C			: std_logic_vector(1 downto 0) := "00";
	constant ABS_ERR_C			: std_logic_vector(1 downto 0) := "01";
	constant REL_ERR_C			: std_logic_vector(1 downto 0) := "10";
	constant ABS_REL_C			: std_logic_vector(1 downto 0) := "11";
	
	-----------------------------------------------------------------------------------------------------------
	-- Values for "ENCODER_TYPE_G"
	constant SMPL_ADAPT_C		: std_logic_vector(1 downto 0) := "00";
	constant HYBRID_C			: std_logic_vector(1 downto 0) := "01";
	constant BLK_ADAPT_C		: std_logic_vector(1 downto 0) := "10";
	
	-----------------------------------------------------------------------------------------------------------
	-- Values for "SUPL_TABLE_TYPE_G"
	constant USG_TYPE_C			: std_logic_vector(1 downto 0) := "00";
	constant SGD_TYPE_C			: std_logic_vector(1 downto 0) := "01";
	constant FLO_TYPE_C			: std_logic_vector(1 downto 0) := "10";
	
	-----------------------------------------------------------------------------------------------------------
	-- Values for "SUPL_TABLE_STRUCT_G"
	constant DIM0_C				: std_logic_vector(1 downto 0) := "00";
	constant DIM1_C				: std_logic_vector(1 downto 0) := "01";
	constant DIM2_ZX_C			: std_logic_vector(1 downto 0) := "10";
	constant DIM2_YX_C			: std_logic_vector(1 downto 0) := "11";
	
	-----------------------------------------------------------------------------------------------------------
	
end package gen_labels;