library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.param_image.all;

-- Package Declaration Section
package types_image is

	-- Record for the image coordinates
	type img_coord_t is record
		x	  : integer range -1 to NX_C-1;
		y	  : integer range -1 to NY_C-1;
		z	  : integer range -1 to NZ_C-1;
		t	  : integer range -NX_C*NY_C to NX_C*NY_C;
	end record img_coord_t;
	
	-- Record for the sample value limits
	type smpl_lim_t is record
		min   : std_logic_vector(D_C-1 downto 0);
		mid   : std_logic_vector(D_C-1 downto 0);
		max	  : std_logic_vector(D_C-1 downto 0);
	end record smpl_lim_t;
	
	-- Record for the float type declaration
	type float_t is record
		b	  : integer range 0 to 1;				-- Supplementary information float table sign bit
		alpha : integer range 0 to (2**DE_C-1);		-- Supplementary information float table exponent
		j	  : integer range 0 to (2**DF_C-1);		-- Supplementary information float table significand
	end record float_t;

	-- Arrays of primitive types
	type array_integer_t	 is array(natural range <>) of integer;
	type array_signed_t		 is array(natural range <>) of signed;
	type array_unsigned_t	 is array(natural range <>) of unsigned;
	type array_slv_t		 is array(natural range <>) of std_logic_vector;
	type array_string_t		 is array(natural range <>) of string;
	-- Arrays of record types
	type array_img_coord_t	 is array(natural range <>) of img_coord_t;
	type array_float_t		 is array(natural range <>) of float_t;
	
	-- Matrixes of primitive types
	type matrix_integer_t	 is array(natural range <>) of array_integer_t;
	type matrix_signed_t	 is array(natural range <>) of array_signed_t;
	type matrix_unsigned_t	 is array(natural range <>) of array_unsigned_t;
	type matrix_slv_t		 is array(natural range <>) of array_slv_t;
	-- Matrixes of record types
	type matrix_float_t		 is array(natural range <>) of array_float_t;

	-- Matrixes 3D of primitive types
	type matrix3D_integer_t	 is array(natural range <>) of matrix_integer_t;
	type matrix3D_signed_t	 is array(natural range <>) of matrix_signed_t;
	type matrix3D_unsigned_t is array(natural range <>) of matrix_unsigned_t;
	type matrix3D_slv_t		 is array(natural range <>) of matrix_slv_t;
	-- Matrixes 3D of record types
	type matrix3D_float_t	 is array(natural range <>) of matrix_float_t;

end package types_image;