library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.param_image.all;
use work.types_image.all;
use work.utils_image.all;

use work.param_predictor.all;
use work.types_predictor.all;

package error_limit_values_table is

	constant A0_ARR_C : array_integer_t(0 to NZ_C-1) := (others => A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R0_ARR_C : array_integer_t(0 to NZ_C-1) := (others => R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array

	constant A1_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 2*A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R1_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 2*R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array
	
	constant A2_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 3*A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R2_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 3*R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array

	constant A3_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 4*A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R3_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 4*R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array
	
	constant A4_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 5*A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R4_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 5*R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array
	
	constant A5_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 6*A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R5_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 6*R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array
	
	constant A6_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 7*A_C);	-- range 0 to (2**DA_C-1). Absolute error limit values array
	constant R6_ARR_C : array_integer_t(0 to NZ_C-1) := (others => 7*R_C);	-- range 0 to (2**DR_C-1). Relative error limit values array
	
	-- Error limit values array defined, with a size depending of the image coordinates (+1 for inital values)
	constant ERR_LIM_ARR_SIZE_C	: integer := (NY_C/(2**U_C)) + 1;
	constant ERR_LIM_VAL_ARR_C	: err_lim_arr_t(0 to ERR_LIM_ARR_SIZE_C-1) := (
		others => (
			abs_c	=> A_C,
			abs_arr	=> A0_ARR_C,
			rel_c	=> R_C,
			rel_arr	=> R0_ARR_C
		)
	);

end package error_limit_values_table;