--------------------------------------------------------------------------------
-- University:	NTNU Trondheim
-- Project:		CCSDS123 Issue 2
-- Engineer:	Cristian Gil Morales
-- Date:		01/11/2020
--------------------------------------------------------------------------------
-- IP name:		predicted_sample
--
-- Description: Predicted sample value "s^z(t)" calculation
--
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.param_image.all;
use work.types_image.all;
use work.utils_image.all;

use work.types_predictor.all;
use work.utils_predictor.all;

entity predicted_sample is
	generic (
		-- 00: BSQ order, 01: BIP order, 10: BIL order
		SMPL_ORDER_G : std_logic_vector(1 downto 0)
	);
	port (
		clock_i		: in  std_logic;
		reset_i		: in  std_logic;
		
		enable_i	: in  std_logic;
		enable_o	: out std_logic;
		img_coord_i	: in  img_coord_t;
		img_coord_o	: out img_coord_t;

		data_s4_i	: in  std_logic_vector(D_C+2-1 downto 0);	-- "s~z(t)" (double-resolution predicted sample)
		data_s3_o	: out std_logic_vector(D_C+1-1 downto 0)	-- "s^z(t)"	(predicted sample)
	);
end entity predicted_sample;

architecture behavioural of predicted_sample is
	signal enable_s		: std_logic   := '0';
	signal img_coord_s	: img_coord_t := reset_img_coord(SMPL_ORDER_G);
	
	signal data_s3_s	: std_logic_vector(D_C+1-1 downto 0) := (others => '0');

begin
	-- Input values delayed to synchronize them with the next modules in chain
	p_pred_smpl_delay : process(clock_i) is
	begin
		if rising_edge(clock_i) then
			if (reset_i = '1') then
				enable_s	<= '0';
				img_coord_s <= reset_img_coord(SMPL_ORDER_G);
			else
				enable_s	<= enable_i;
				img_coord_s	<= img_coord_i;
			end if;
		end if;
	end process p_pred_smpl_delay;
	
	-- Predicted sample (s^z(t)) calculation
	p_pred_smpl_calc : process(clock_i) is
	begin
		if rising_edge(clock_i) then
			if (reset_i = '1') then
				data_s3_s <= (others => '0');
			else
				if (enable_i = '1') then
					data_s3_s <= std_logic_vector(resize(shift_sgn(signed(data_s4_i), -1), D_C+1));
				end if;
			end if;
		end if;
	end process p_pred_smpl_calc;

	-- Outputs
	enable_o	<= enable_s;
	img_coord_o	<= img_coord_s;
	data_s3_o	<= data_s3_s;
end architecture behavioural;