#/bin/sh

# ***********************************************************************************
# ******************* EXECUTION OF ALL EXISTING VUnit TESTBENCHES *******************
# ***********************************************************************************

# ***********************************************************************************
# INSTRUCTIONS FOR EXECUTION: 
#	1. Install "Python3" (including it to the PATH as well).
#	2. Install "pip" (python packages installer)
#	3. Install "VUnit" package: --> "pip install vunit_hdl"
#	4. Open a terminal and execute the present file --> "sh run_simulations.sh"
#	5. Follow instructions
# ***********************************************************************************

# VUnit environment variables to limit the path characters, so that the testbenches can be executed everywhere.
# On Windows, path with more than 260 characters are not accepted...
export VUNIT_SHORT_TEST_OUTPUT_PATHS=200
export VUNIT_TEST_OUTPUT_PATH_MARGIN=200

# Displays a bunch of validation options, reads back the argument, and repeats if wrong value
echo -e -n "
Select a block to validate:
  - Top entity:			0
  - Image block:		1
  - Predictor block:		2
     - Adder sub-block:		3
     - Quantizer sub-block:	4
     - Mapper sub-block:	5
     - Sample Repr. sub-block: 	6
     - Prediction sub-block:	7
  - Encoder block:		8
     - Enc. Header sub-block:	9
     - Enc. Body sub-block:	10
        - Sample-Adapt. Coder:	11
        - Hybrid Coder:		12
     - Packer sub-block:	13
  - All blocks:			14
Clean compiled files:		15
\n"

# Infinite loop until a number from 0 to 15 is entered
unset option
until [[ $option == +([0-9]) && $option -le 15 ]]; do
    read -r -p "Enter a valid number: " option
done

# Defines path and folder name, depending on the previous selection
unset path
unset place
if [[ $option -eq 0 ]]; then
	path="./TOP/"; place="TOP"
elif [[ $option -eq 1 ]]; then
	path="./Image/"; place="IMAGE"
elif [[ $option -eq 2 ]]; then
	path="./Predictor/"; place="PREDICTOR"
elif [[ $option -eq 3 ]]; then
	path="./Predictor/adder/"; place="PRED: ADDER"
elif [[ $option -eq 4 ]]; then
	path="./Predictor/quantizer/"; place="PRED: QUANTIZER"
elif [[ $option -eq 5 ]]; then
	path="./Predictor/mapper/"; place="PRED: MAPPER"
elif [[ $option -eq 6 ]]; then
	path="./Predictor/sample_representative"; place="PRED: SAMPLE REPR."
elif [[ $option -eq 7 ]]; then
	path="./Predictor/prediction/"; place="PRED: PREDICTION"
elif [[ $option -eq 8 ]]; then
	path="./Encoder/"; place="ENCODER"
elif [[ $option -eq 9 ]]; then
	path="./Encoder/header"; place="ENC: ENCODER HEADER"
elif [[ $option -eq 10 ]]; then
	path="./Encoder/body"; place="ENC: ENCODER BODY"
elif [[ $option -eq 11 ]]; then
	path="./Encoder/body/sample_adaptive_coder"; place="ENC. BODY: SMPL-ADAPT. CODER"
elif [[ $option -eq 12 ]]; then
	path="./Encoder/body/hybrid_coder"; place="ENC. BODY: HYBRID CODER"
elif [[ $option -eq 13 ]]; then
	path="./Encoder/packer"; place="ENC: PACKER"
elif [[ $option -eq 14 ]]; then
	path="."; place="ALL"
else
	rm -rf vunit_out/;
	echo -e "\n --> FOLDER 'vunit_out' DELETED. EXITING... ";
	exit 0;
fi

echo -e "\n ******************************************************* "
echo -e " *** VALIDATING -> $place IP(s) "
echo -e " ******************************************************* "

# Looking (on the chosen path, and recursively) for all VUnit python files and execute them
for file in $(find $path -name 'run_*.py'); do
	echo -e "\n --> EXECUTING TESTBENCH \"${file#*simulation/}\" \n"
	
	# Executes all python files, but if any of them fails, then informs and exits the loop
	python $file ||
		{ echo -e "\n PROBLEM FOUND ON TESTBENCH \"${file#*simulation/}\". VALIDATION PROCESS ABORTED..."; exit 1; }
done
