library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.types_image.all;

-- Package Declaration Section
package hybrid_code_tables is
	
	-- Hybrid Code Table record (and its array) types
	type hybrid_code_table_t is record
		input_code	: array_string_t(0 to 256)(1 to 14);
		output_code	: array_integer_t(0 to 256);
		output_size	: array_integer_t(0 to 256);
		table_size	: integer;
	end record hybrid_code_table_t;
	type hybrid_code_tables_arr_t is array(natural range <>) of hybrid_code_table_t;
	
	constant HYBRID_CODE_TABLE0_C  : hybrid_code_table_t := (
		input_code  => ("0^{002}       ", "0^{001}10     ", "0^{001}11     ", "0^{001}12     ", "0^{001}13     ", "0^{001}14     ", "0^{001}15     ", "0^{001}16     ", "0^{001}17     ", "0^{001}18     ", "0^{001}19     ", "0^{001}1A     ", "0^{001}1B     ", "0^{001}1C     ", "0^{001}1X     ", "0^{001}20     ", "0^{001}21     ", "0^{001}22     ", "0^{001}23     ", "0^{001}24     ", "0^{001}25     ", "0^{001}26     ", "0^{001}27     ", "0^{001}28     ", "0^{001}29     ", "0^{001}2A     ", "0^{001}2B     ", "0^{001}2C     ", "0^{001}2X     ", "0^{001}3      ", "0^{001}4      ", "0^{001}5      ", "0^{001}6      ", "0^{001}7      ", "0^{001}8      ", "0^{001}9      ", "0^{001}A      ", "0^{001}B      ", "0^{001}C      ", "0^{001}X      ", "0^{000}1      ", "0^{000}2      ", "0^{000}3      ", "0^{000}40     ", "0^{000}41     ", "0^{000}42     ", "0^{000}43     ", "0^{000}440    ", "0^{000}441    ", "0^{000}442    ", "0^{000}443    ", "0^{000}444    ", "0^{000}445    ", "0^{000}446    ", "0^{000}447    ", "0^{000}448    ", "0^{000}449    ", "0^{000}44A    ", "0^{000}44B    ", "0^{000}44C    ", "0^{000}44X    ", "0^{000}45     ", "0^{000}46     ", "0^{000}47     ", "0^{000}48     ", "0^{000}49     ", "0^{000}4A     ", "0^{000}4B     ", "0^{000}4C     ", "0^{000}4X     ", "0^{000}5      ", "0^{000}6      ", "0^{000}70     ", "0^{000}71     ", "0^{000}72     ", "0^{000}73     ", "0^{000}74     ", "0^{000}75     ", "0^{000}76     ", "0^{000}77     ", "0^{000}78     ", "0^{000}79     ", "0^{000}7A     ", "0^{000}7B     ", "0^{000}7C     ", "0^{000}7X     ", "0^{000}80     ", "0^{000}81     ", "0^{000}82     ", "0^{000}83     ", "0^{000}84     ", "0^{000}85     ", "0^{000}86     ", "0^{000}87     ", "0^{000}88     ", "0^{000}89     ", "0^{000}8A     ", "0^{000}8B     ", "0^{000}8C     ", "0^{000}8X     ", "0^{000}9      ", "0^{000}A      ", "0^{000}B      ", "0^{000}C      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#19#, 16#57#, 16#d7#, 16#37#, 16#af#, 16#1af#, 16#6f#, 16#16f#, 16#3f#, 16#23f#, 16#17f#, 16#57f#, 16#1ff#, 16#9ff#, 16#37f#, 16#b7#, 16#77#, 16#f7#, 16#ef#, 16#1ef#, 16#1f#, 16#11f#, 16#13f#, 16#33f#, 16#77f#, 16#ff#, 16#5ff#, 16#dff#, 16#4ff#, 16#15#, 16#35#, 16#d#, 16#2d#, 16#13#, 16#53#, 16#3b#, 16#bb#, 16#f#, 16#10f#, 16#7b#, 16#0#, 16#4#, 16#2#, 16#1d#, 16#3d#, 16#3#, 16#23#, 16#9f#, 16#19f#, 16#5f#, 16#bf#, 16#2bf#, 16#1bf#, 16#3bf#, 16#2ff#, 16#6ff#, 16#3ff#, 16#bff#, 16#fff#, 16#1fff#, 16#7ff#, 16#33#, 16#73#, 16#fb#, 16#7#, 16#8f#, 16#18f#, 16#4f#, 16#14f#, 16#87#, 16#6#, 16#e#, 16#b#, 16#4b#, 16#2b#, 16#47#, 16#c7#, 16#27#, 16#a7#, 16#cf#, 16#1cf#, 16#15f#, 16#35f#, 16#7f#, 16#47f#, 16#df#, 16#6b#, 16#1b#, 16#5b#, 16#67#, 16#e7#, 16#17#, 16#97#, 16#2f#, 16#12f#, 16#2df#, 16#1df#, 16#27f#, 16#67f#, 16#3df#, 16#1#, 16#11#, 16#5#, 16#25#, 16#9#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (5, 8, 8, 8, 9, 9, 9, 9, 10, 10, 11, 11, 12, 12, 11, 8, 8, 8, 9, 9, 9, 9, 10, 10, 11, 11, 12, 12, 11, 6, 6, 6, 6, 7, 7, 8, 8, 9, 9, 8, 3, 3, 3, 6, 6, 6, 6, 9, 9, 9, 10, 10, 10, 10, 11, 11, 12, 12, 13, 13, 12, 7, 7, 8, 8, 9, 9, 9, 9, 8, 4, 4, 7, 7, 7, 8, 8, 8, 8, 9, 9, 10, 10, 11, 11, 10, 7, 7, 7, 8, 8, 8, 8, 9, 9, 10, 10, 11, 11, 10, 5, 5, 6, 6, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 105
	);
	
	constant HYBRID_CODE_TABLE1_C  : hybrid_code_table_t := (
		input_code  => ("0^{003}       ", "0^{002}1      ", "0^{002}2      ", "0^{002}3      ", "0^{002}4      ", "0^{002}5      ", "0^{002}6      ", "0^{002}7      ", "0^{002}8      ", "0^{002}9      ", "0^{002}A      ", "0^{002}X      ", "0^{001}1      ", "0^{001}2      ", "0^{001}30     ", "0^{001}31     ", "0^{001}32     ", "0^{001}33     ", "0^{001}34     ", "0^{001}35     ", "0^{001}36     ", "0^{001}37     ", "0^{001}38     ", "0^{001}39     ", "0^{001}3A     ", "0^{001}3X     ", "0^{001}40     ", "0^{001}41     ", "0^{001}42     ", "0^{001}43     ", "0^{001}44     ", "0^{001}45     ", "0^{001}46     ", "0^{001}47     ", "0^{001}48     ", "0^{001}49     ", "0^{001}4A     ", "0^{001}4X     ", "0^{001}5      ", "0^{001}6      ", "0^{001}7      ", "0^{001}8      ", "0^{001}9      ", "0^{001}A      ", "0^{001}X      ", "0^{000}10     ", "0^{000}11     ", "0^{000}12     ", "0^{000}130    ", "0^{000}131    ", "0^{000}132    ", "0^{000}133    ", "0^{000}134    ", "0^{000}135    ", "0^{000}136    ", "0^{000}137    ", "0^{000}138    ", "0^{000}139    ", "0^{000}13A    ", "0^{000}13X    ", "0^{000}14     ", "0^{000}150    ", "0^{000}151    ", "0^{000}152    ", "0^{000}153    ", "0^{000}154    ", "0^{000}155    ", "0^{000}156    ", "0^{000}157    ", "0^{000}158    ", "0^{000}159    ", "0^{000}15A    ", "0^{000}15X    ", "0^{000}160    ", "0^{000}161    ", "0^{000}162    ", "0^{000}163    ", "0^{000}164    ", "0^{000}165    ", "0^{000}166    ", "0^{000}167    ", "0^{000}168    ", "0^{000}169    ", "0^{000}16A    ", "0^{000}16X    ", "0^{000}17     ", "0^{000}18     ", "0^{000}19     ", "0^{000}1A     ", "0^{000}1X     ", "0^{000}20     ", "0^{000}21     ", "0^{000}22     ", "0^{000}230    ", "0^{000}231    ", "0^{000}232    ", "0^{000}233    ", "0^{000}234    ", "0^{000}235    ", "0^{000}236    ", "0^{000}237    ", "0^{000}238    ", "0^{000}239    ", "0^{000}23A    ", "0^{000}23X    ", "0^{000}24     ", "0^{000}250    ", "0^{000}251    ", "0^{000}252    ", "0^{000}253    ", "0^{000}254    ", "0^{000}255    ", "0^{000}256    ", "0^{000}257    ", "0^{000}258    ", "0^{000}259    ", "0^{000}25A    ", "0^{000}25X    ", "0^{000}26     ", "0^{000}27     ", "0^{000}28     ", "0^{000}29     ", "0^{000}2A     ", "0^{000}2X     ", "0^{000}3      ", "0^{000}4      ", "0^{000}5      ", "0^{000}6      ", "0^{000}7      ", "0^{000}8      ", "0^{000}9      ", "0^{000}A0     ", "0^{000}A1     ", "0^{000}A2     ", "0^{000}A3     ", "0^{000}A4     ", "0^{000}A5     ", "0^{000}A6     ", "0^{000}A7     ", "0^{000}A8     ", "0^{000}A9     ", "0^{000}AA     ", "0^{000}AX     ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#73#, 16#b#, 16#4b#, 16#9b#, 16#5b#, 16#137#, 16#b7#, 16#11f#, 16#31f#, 16#63f#, 16#13f#, 16#53f#, 16#e#, 16#1e#, 16#db#, 16#3b#, 16#bb#, 16#1b7#, 16#77#, 16#177#, 16#f7#, 16#9f#, 16#33f#, 16#57f#, 16#d7f#, 16#73f#, 16#7b#, 16#fb#, 16#7#, 16#1f7#, 16#f#, 16#10f#, 16#29f#, 16#bf#, 16#4bf#, 16#37f#, 16#b7f#, 16#2bf#, 16#1d#, 16#3d#, 16#13#, 16#53#, 16#e7#, 16#1e7#, 16#2b#, 16#1#, 16#11#, 16#9#, 16#87#, 16#47#, 16#c7#, 16#8f#, 16#18f#, 16#19f#, 16#39f#, 16#6bf#, 16#1bf#, 16#77f#, 16#f7f#, 16#ff#, 16#3#, 16#4f#, 16#14f#, 16#cf#, 16#5f#, 16#25f#, 16#15f#, 16#5bf#, 16#8ff#, 16#4ff#, 16#3ff#, 16#13ff#, 16#cff#, 16#1cf#, 16#2f#, 16#12f#, 16#35f#, 16#df#, 16#3bf#, 16#7bf#, 16#2ff#, 16#aff#, 16#bff#, 16#1bff#, 16#7ff#, 16#ab#, 16#6b#, 16#17#, 16#117#, 16#97#, 16#19#, 16#5#, 16#15#, 16#27#, 16#a7#, 16#67#, 16#af#, 16#1af#, 16#2df#, 16#1df#, 16#7f#, 16#47f#, 16#6ff#, 16#eff#, 16#1ff#, 16#23#, 16#6f#, 16#16f#, 16#ef#, 16#3df#, 16#3f#, 16#27f#, 16#67f#, 16#9ff#, 16#5ff#, 16#17ff#, 16#fff#, 16#1fff#, 16#33#, 16#eb#, 16#1b#, 16#197#, 16#57#, 16#157#, 16#0#, 16#4#, 16#2#, 16#a#, 16#6#, 16#16#, 16#d#, 16#d7#, 16#1d7#, 16#37#, 16#1ef#, 16#3ef#, 16#1f#, 16#21f#, 16#23f#, 16#17f#, 16#dff#, 16#1dff#, 16#97f#, 16#2d#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (7, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 11, 5, 5, 8, 8, 8, 9, 9, 9, 9, 10, 11, 12, 12, 11, 8, 8, 8, 9, 9, 9, 10, 11, 11, 12, 12, 11, 6, 6, 7, 7, 9, 9, 8, 5, 5, 5, 8, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 12, 6, 9, 9, 9, 10, 10, 10, 11, 12, 12, 13, 13, 12, 9, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13, 13, 8, 8, 9, 9, 9, 5, 5, 5, 8, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 12, 6, 9, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13, 13, 7, 8, 8, 9, 9, 9, 3, 3, 4, 4, 5, 5, 6, 9, 9, 9, 10, 10, 10, 10, 11, 12, 13, 13, 12, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 144
	);
	
	constant HYBRID_CODE_TABLE2_C  : hybrid_code_table_t := (
		input_code  => ("0^{001}       ", "0^{000}10     ", "0^{000}110    ", "0^{000}111    ", "0^{000}112    ", "0^{000}113    ", "0^{000}114    ", "0^{000}115    ", "0^{000}116    ", "0^{000}117    ", "0^{000}118    ", "0^{000}11X    ", "0^{000}120    ", "0^{000}121    ", "0^{000}122    ", "0^{000}123    ", "0^{000}124    ", "0^{000}125    ", "0^{000}126    ", "0^{000}127    ", "0^{000}128    ", "0^{000}12X    ", "0^{000}130    ", "0^{000}131    ", "0^{000}132    ", "0^{000}133    ", "0^{000}134    ", "0^{000}135    ", "0^{000}136    ", "0^{000}137    ", "0^{000}138    ", "0^{000}13X    ", "0^{000}140    ", "0^{000}141    ", "0^{000}142    ", "0^{000}143    ", "0^{000}144    ", "0^{000}145    ", "0^{000}146    ", "0^{000}147    ", "0^{000}148    ", "0^{000}14X    ", "0^{000}15     ", "0^{000}16     ", "0^{000}17     ", "0^{000}18     ", "0^{000}1X     ", "0^{000}200    ", "0^{000}201    ", "0^{000}202    ", "0^{000}203    ", "0^{000}204    ", "0^{000}205    ", "0^{000}206    ", "0^{000}207    ", "0^{000}208    ", "0^{000}20X    ", "0^{000}210    ", "0^{000}211    ", "0^{000}212    ", "0^{000}213    ", "0^{000}214    ", "0^{000}215    ", "0^{000}216    ", "0^{000}217    ", "0^{000}218    ", "0^{000}21X    ", "0^{000}22     ", "0^{000}230    ", "0^{000}231    ", "0^{000}232    ", "0^{000}233    ", "0^{000}234    ", "0^{000}235    ", "0^{000}236    ", "0^{000}237    ", "0^{000}238    ", "0^{000}23X    ", "0^{000}240    ", "0^{000}241    ", "0^{000}242    ", "0^{000}243    ", "0^{000}244    ", "0^{000}245    ", "0^{000}246    ", "0^{000}247    ", "0^{000}248    ", "0^{000}24X    ", "0^{000}25     ", "0^{000}26     ", "0^{000}27     ", "0^{000}28     ", "0^{000}2X     ", "0^{000}3      ", "0^{000}4      ", "0^{000}50     ", "0^{000}51     ", "0^{000}52     ", "0^{000}53     ", "0^{000}54     ", "0^{000}55     ", "0^{000}56     ", "0^{000}57     ", "0^{000}58     ", "0^{000}5X     ", "0^{000}60     ", "0^{000}61     ", "0^{000}62     ", "0^{000}63     ", "0^{000}64     ", "0^{000}65     ", "0^{000}66     ", "0^{000}67     ", "0^{000}68     ", "0^{000}6X     ", "0^{000}7      ", "0^{000}8      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#1#, 16#7d#, 16#3#, 16#43#, 16#3b#, 16#bb#, 16#cf#, 16#1cf#, 16#3bf#, 16#7bf#, 16#7f#, 16#23#, 16#63#, 16#13#, 16#7b#, 16#fb#, 16#2f#, 16#12f#, 16#47f#, 16#27f#, 16#67f#, 16#53#, 16#7#, 16#87#, 16#af#, 16#1af#, 16#df#, 16#2df#, 16#2ff#, 16#aff#, 16#6ff#, 16#47#, 16#c7#, 16#27#, 16#6f#, 16#16f#, 16#1df#, 16#3df#, 16#eff#, 16#1ff#, 16#9ff#, 16#35#, 16#75#, 16#6b#, 16#f7#, 16#1f7#, 16#15#, 16#33#, 16#73#, 16#a7#, 16#67#, 16#ef#, 16#1ef#, 16#17f#, 16#57f#, 16#37f#, 16#b#, 16#4b#, 16#2b#, 16#e7#, 16#17#, 16#1f#, 16#11f#, 16#77f#, 16#ff#, 16#4ff#, 16#9#, 16#97#, 16#57#, 16#d7#, 16#9f#, 16#19f#, 16#3f#, 16#23f#, 16#5ff#, 16#dff#, 16#3ff#, 16#37#, 16#b7#, 16#77#, 16#5f#, 16#15f#, 16#13f#, 16#33f#, 16#bff#, 16#7ff#, 16#fff#, 16#d#, 16#4d#, 16#eb#, 16#f#, 16#10f#, 16#2#, 16#6#, 16#25#, 16#2d#, 16#6d#, 16#1b#, 16#9b#, 16#8f#, 16#18f#, 16#bf#, 16#4bf#, 16#2bf#, 16#1d#, 16#5d#, 16#3d#, 16#5b#, 16#db#, 16#4f#, 16#14f#, 16#6bf#, 16#1bf#, 16#5bf#, 16#19#, 16#39#, 16#5#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (2, 4, 7, 7, 7, 8, 8, 9, 9, 11, 11, 11, 7, 7, 7, 8, 8, 9, 9, 11, 11, 11, 7, 8, 8, 9, 9, 10, 10, 12, 12, 12, 8, 8, 8, 9, 9, 10, 10, 12, 12, 12, 7, 7, 8, 9, 9, 6, 7, 7, 8, 8, 9, 9, 11, 11, 11, 7, 7, 7, 8, 8, 9, 9, 11, 11, 11, 5, 8, 8, 8, 9, 9, 10, 10, 12, 12, 12, 8, 8, 8, 9, 9, 10, 10, 12, 12, 12, 7, 7, 8, 9, 9, 3, 3, 6, 7, 7, 8, 8, 9, 9, 11, 11, 11, 7, 7, 7, 8, 8, 9, 9, 11, 11, 11, 6, 6, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 118
	);
	
	constant HYBRID_CODE_TABLE3_C  : hybrid_code_table_t := (
		input_code  => ("0^{003}       ", "0^{002}10     ", "0^{002}11     ", "0^{002}12     ", "0^{002}13     ", "0^{002}14     ", "0^{002}15     ", "0^{002}16     ", "0^{002}1X     ", "0^{002}20     ", "0^{002}21     ", "0^{002}22     ", "0^{002}23     ", "0^{002}24     ", "0^{002}25     ", "0^{002}26     ", "0^{002}2X     ", "0^{002}3      ", "0^{002}4      ", "0^{002}5      ", "0^{002}6      ", "0^{002}X      ", "0^{001}1      ", "0^{001}2      ", "0^{001}3      ", "0^{001}4      ", "0^{001}5      ", "0^{001}6      ", "0^{001}X      ", "0^{000}1      ", "0^{000}20     ", "0^{000}210    ", "0^{000}2110   ", "0^{000}2111   ", "0^{000}2112   ", "0^{000}2113   ", "0^{000}2114   ", "0^{000}2115   ", "0^{000}2116   ", "0^{000}211X   ", "0^{000}212    ", "0^{000}213    ", "0^{000}214    ", "0^{000}215    ", "0^{000}216    ", "0^{000}21X    ", "0^{000}220    ", "0^{000}2210   ", "0^{000}2211   ", "0^{000}2212   ", "0^{000}2213   ", "0^{000}2214   ", "0^{000}2215   ", "0^{000}2216   ", "0^{000}221X   ", "0^{000}222    ", "0^{000}223    ", "0^{000}224    ", "0^{000}225    ", "0^{000}226    ", "0^{000}22X    ", "0^{000}230    ", "0^{000}231    ", "0^{000}232    ", "0^{000}233    ", "0^{000}234    ", "0^{000}235    ", "0^{000}236    ", "0^{000}23X    ", "0^{000}24     ", "0^{000}25     ", "0^{000}26     ", "0^{000}2X     ", "0^{000}30     ", "0^{000}310    ", "0^{000}311    ", "0^{000}312    ", "0^{000}313    ", "0^{000}314    ", "0^{000}315    ", "0^{000}316    ", "0^{000}31X    ", "0^{000}320    ", "0^{000}321    ", "0^{000}322    ", "0^{000}323    ", "0^{000}324    ", "0^{000}325    ", "0^{000}326    ", "0^{000}32X    ", "0^{000}33     ", "0^{000}34     ", "0^{000}35     ", "0^{000}36     ", "0^{000}3X     ", "0^{000}40     ", "0^{000}410    ", "0^{000}411    ", "0^{000}412    ", "0^{000}413    ", "0^{000}414    ", "0^{000}415    ", "0^{000}416    ", "0^{000}41X    ", "0^{000}42     ", "0^{000}43     ", "0^{000}44     ", "0^{000}45     ", "0^{000}46     ", "0^{000}4X     ", "0^{000}5      ", "0^{000}60     ", "0^{000}61     ", "0^{000}62     ", "0^{000}63     ", "0^{000}64     ", "0^{000}65     ", "0^{000}66     ", "0^{000}6X     ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#19#, 16#4b#, 16#17#, 16#97#, 16#ef#, 16#1ef#, 16#37f#, 16#77f#, 16#9ff#, 16#2b#, 16#57#, 16#d7#, 16#1f#, 16#11f#, 16#ff#, 16#4ff#, 16#5ff#, 16#43#, 16#23#, 16#18f#, 16#4f#, 16#14f#, 16#2#, 16#a#, 16#1e#, 16#1#, 16#2d#, 16#6d#, 16#6b#, 16#0#, 16#6#, 16#35#, 16#37#, 16#9f#, 16#19f#, 16#23f#, 16#13f#, 16#dff#, 16#3ff#, 16#fff#, 16#63#, 16#3b#, 16#bb#, 16#2df#, 16#1df#, 16#1bf#, 16#d#, 16#b7#, 16#5f#, 16#15f#, 16#33f#, 16#bf#, 16#bff#, 16#7ff#, 16#1fff#, 16#13#, 16#7b#, 16#fb#, 16#3df#, 16#3f#, 16#5bf#, 16#53#, 16#7#, 16#87#, 16#cf#, 16#1cf#, 16#3bf#, 16#7bf#, 16#aff#, 16#25#, 16#eb#, 16#1b#, 16#9b#, 16#11#, 16#33#, 16#47#, 16#c7#, 16#2f#, 16#12f#, 16#7f#, 16#47f#, 16#6ff#, 16#73#, 16#27#, 16#a7#, 16#af#, 16#1af#, 16#27f#, 16#67f#, 16#eff#, 16#1d#, 16#5d#, 16#77#, 16#177#, 16#f7#, 16#9#, 16#b#, 16#67#, 16#e7#, 16#6f#, 16#16f#, 16#17f#, 16#57f#, 16#1ff#, 16#15#, 16#3d#, 16#7d#, 16#1f7#, 16#f#, 16#df#, 16#e#, 16#3#, 16#5b#, 16#db#, 16#10f#, 16#8f#, 16#2bf#, 16#6bf#, 16#2ff#, 16#5#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (5, 7, 8, 8, 9, 9, 11, 11, 12, 7, 8, 8, 9, 9, 11, 11, 12, 7, 7, 9, 9, 9, 4, 4, 5, 5, 7, 7, 8, 2, 4, 6, 8, 9, 9, 10, 10, 12, 12, 13, 7, 8, 8, 10, 10, 11, 6, 8, 9, 9, 10, 10, 12, 12, 13, 7, 8, 8, 10, 10, 11, 7, 8, 8, 9, 9, 11, 11, 12, 6, 8, 8, 8, 5, 7, 8, 8, 9, 9, 11, 11, 12, 7, 8, 8, 9, 9, 11, 11, 12, 7, 7, 9, 9, 9, 5, 7, 8, 8, 9, 9, 11, 11, 12, 6, 7, 7, 9, 9, 10, 5, 7, 8, 8, 9, 9, 11, 11, 12, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 120
	);
	
	constant HYBRID_CODE_TABLE4_C  : hybrid_code_table_t := (
		input_code  => ("0^{003}       ", "0^{002}10     ", "0^{002}11     ", "0^{002}12     ", "0^{002}13     ", "0^{002}14     ", "0^{002}15     ", "0^{002}16     ", "0^{002}1X     ", "0^{002}2      ", "0^{002}3      ", "0^{002}4      ", "0^{002}5      ", "0^{002}6      ", "0^{002}X      ", "0^{001}100    ", "0^{001}101    ", "0^{001}102    ", "0^{001}103    ", "0^{001}104    ", "0^{001}105    ", "0^{001}106    ", "0^{001}10X    ", "0^{001}110    ", "0^{001}111    ", "0^{001}112    ", "0^{001}113    ", "0^{001}114    ", "0^{001}115    ", "0^{001}116    ", "0^{001}11X    ", "0^{001}120    ", "0^{001}121    ", "0^{001}122    ", "0^{001}123    ", "0^{001}124    ", "0^{001}125    ", "0^{001}126    ", "0^{001}12X    ", "0^{001}13     ", "0^{001}14     ", "0^{001}15     ", "0^{001}16     ", "0^{001}1X     ", "0^{001}20     ", "0^{001}210    ", "0^{001}211    ", "0^{001}212    ", "0^{001}213    ", "0^{001}214    ", "0^{001}215    ", "0^{001}216    ", "0^{001}21X    ", "0^{001}220    ", "0^{001}221    ", "0^{001}222    ", "0^{001}223    ", "0^{001}224    ", "0^{001}225    ", "0^{001}226    ", "0^{001}22X    ", "0^{001}23     ", "0^{001}24     ", "0^{001}25     ", "0^{001}26     ", "0^{001}2X     ", "0^{001}30     ", "0^{001}31     ", "0^{001}32     ", "0^{001}33     ", "0^{001}34     ", "0^{001}35     ", "0^{001}36     ", "0^{001}3X     ", "0^{001}40     ", "0^{001}41     ", "0^{001}42     ", "0^{001}43     ", "0^{001}44     ", "0^{001}45     ", "0^{001}46     ", "0^{001}4X     ", "0^{001}5      ", "0^{001}6      ", "0^{001}X      ", "0^{000}1      ", "0^{000}2      ", "0^{000}3      ", "0^{000}4      ", "0^{000}5      ", "0^{000}6      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#5#, 16#3#, 16#2b#, 16#6b#, 16#11f#, 16#9f#, 16#8ff#, 16#4ff#, 16#cff#, 16#d#, 16#33#, 16#73#, 16#1ef#, 16#15f#, 16#35f#, 16#23#, 16#1b#, 16#5b#, 16#19f#, 16#5f#, 16#2ff#, 16#aff#, 16#6ff#, 16#3b#, 16#f7#, 16#f#, 16#13f#, 16#33f#, 16#eff#, 16#1dff#, 16#3ff#, 16#7b#, 16#8f#, 16#4f#, 16#bf#, 16#2bf#, 16#1ff#, 16#13ff#, 16#bff#, 16#e7#, 16#17#, 16#df#, 16#2df#, 16#17f#, 16#1d#, 16#7#, 16#cf#, 16#2f#, 16#1bf#, 16#3bf#, 16#9ff#, 16#1bff#, 16#7ff#, 16#47#, 16#af#, 16#6f#, 16#7f#, 16#27f#, 16#17ff#, 16#fff#, 16#1fff#, 16#97#, 16#57#, 16#1df#, 16#57f#, 16#37f#, 16#b#, 16#d7#, 16#37#, 16#1f#, 16#3df#, 16#77f#, 16#f7f#, 16#5ff#, 16#4b#, 16#b7#, 16#77#, 16#3f#, 16#23f#, 16#ff#, 16#15ff#, 16#dff#, 16#a7#, 16#67#, 16#ef#, 16#0#, 16#2#, 16#1#, 16#9#, 16#13#, 16#53#, 16#27#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (4, 6, 7, 7, 9, 9, 12, 12, 12, 5, 7, 7, 9, 10, 10, 6, 7, 7, 9, 9, 12, 12, 12, 7, 8, 8, 10, 10, 12, 13, 13, 7, 8, 8, 10, 10, 12, 13, 13, 8, 8, 10, 10, 11, 5, 7, 8, 8, 10, 10, 12, 13, 13, 7, 8, 8, 10, 10, 13, 13, 13, 8, 8, 10, 11, 11, 7, 8, 8, 9, 10, 12, 12, 13, 7, 8, 8, 10, 10, 12, 13, 13, 8, 8, 9, 2, 2, 4, 4, 7, 7, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 92
	);
	
	constant HYBRID_CODE_TABLE5_C  : hybrid_code_table_t := (
		input_code  => ("0^{001}       ", "0^{000}1      ", "0^{000}2000   ", "0^{000}200100 ", "0^{000}200101 ", "0^{000}200102 ", "0^{000}200103 ", "0^{000}200104 ", "0^{000}20010X ", "0^{000}20011  ", "0^{000}20012  ", "0^{000}20013  ", "0^{000}20014  ", "0^{000}2001X  ", "0^{000}20020  ", "0^{000}20021  ", "0^{000}20022  ", "0^{000}20023  ", "0^{000}20024  ", "0^{000}2002X  ", "0^{000}2003   ", "0^{000}2004   ", "0^{000}200X   ", "0^{000}201000 ", "0^{000}201001 ", "0^{000}201002 ", "0^{000}201003 ", "0^{000}201004 ", "0^{000}20100X ", "0^{000}20101  ", "0^{000}20102  ", "0^{000}20103  ", "0^{000}20104  ", "0^{000}2010X  ", "0^{000}2011   ", "0^{000}2012   ", "0^{000}2013   ", "0^{000}2014   ", "0^{000}201X   ", "0^{000}20200  ", "0^{000}20201  ", "0^{000}20202  ", "0^{000}20203  ", "0^{000}20204  ", "0^{000}2020X  ", "0^{000}2021   ", "0^{000}2022   ", "0^{000}2023   ", "0^{000}2024   ", "0^{000}202X   ", "0^{000}203    ", "0^{000}204    ", "0^{000}20X    ", "0^{000}210000 ", "0^{000}210001 ", "0^{000}210002 ", "0^{000}210003 ", "0^{000}210004 ", "0^{000}21000X ", "0^{000}21001  ", "0^{000}21002  ", "0^{000}21003  ", "0^{000}21004  ", "0^{000}2100X  ", "0^{000}2101   ", "0^{000}2102   ", "0^{000}2103   ", "0^{000}2104   ", "0^{000}210X   ", "0^{000}2110   ", "0^{000}2111   ", "0^{000}2112   ", "0^{000}2113   ", "0^{000}2114   ", "0^{000}211X   ", "0^{000}2120   ", "0^{000}2121   ", "0^{000}2122   ", "0^{000}2123   ", "0^{000}2124   ", "0^{000}212X   ", "0^{000}213    ", "0^{000}214    ", "0^{000}21X    ", "0^{000}22000  ", "0^{000}22001  ", "0^{000}22002  ", "0^{000}22003  ", "0^{000}22004  ", "0^{000}2200X  ", "0^{000}2201   ", "0^{000}2202   ", "0^{000}2203   ", "0^{000}2204   ", "0^{000}220X   ", "0^{000}2210   ", "0^{000}2211   ", "0^{000}2212   ", "0^{000}2213   ", "0^{000}2214   ", "0^{000}221X   ", "0^{000}222    ", "0^{000}223    ", "0^{000}224    ", "0^{000}22X    ", "0^{000}23     ", "0^{000}240    ", "0^{000}241    ", "0^{000}242    ", "0^{000}243    ", "0^{000}244    ", "0^{000}24X    ", "0^{000}2X     ", "0^{000}3      ", "0^{000}4      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#1#, 16#b#, 16#1f#, 16#1df#, 16#3df#, 16#5ff#, 16#15ff#, 16#1fff#, 16#4f#, 16#14f#, 16#d7f#, 16#37f#, 16#7ff#, 16#57#, 16#cf#, 16#1cf#, 16#b7f#, 16#77f#, 16#27ff#, 16#177#, 16#2df#, 16#4bf#, 16#11f#, 16#3f#, 16#23f#, 16#dff#, 16#1dff#, 16#3fff#, 16#2f#, 16#12f#, 16#f7f#, 16#ff#, 16#17ff#, 16#fb#, 16#7#, 16#2bf#, 16#6bf#, 16#1ff#, 16#d7#, 16#af#, 16#1af#, 16#8ff#, 16#4ff#, 16#37ff#, 16#87#, 16#47#, 16#1bf#, 16#5bf#, 16#11ff#, 16#7b#, 16#1b7#, 16#19f#, 16#9f#, 16#13f#, 16#33f#, 16#3ff#, 16#13ff#, 16#7fff#, 16#6f#, 16#16f#, 16#cff#, 16#2ff#, 16#fff#, 16#c7#, 16#27#, 16#3bf#, 16#7bf#, 16#9ff#, 16#a7#, 16#f7#, 16#1f7#, 16#a7f#, 16#67f#, 16#2bff#, 16#67#, 16#f#, 16#10f#, 16#e7f#, 16#17f#, 16#1bff#, 16#39f#, 16#5f#, 16#bf#, 16#37#, 16#ef#, 16#1ef#, 16#aff#, 16#6ff#, 16#2fff#, 16#e7#, 16#17#, 16#7f#, 16#47f#, 16#19ff#, 16#97#, 16#8f#, 16#18f#, 16#97f#, 16#57f#, 16#3bff#, 16#3b#, 16#25f#, 16#15f#, 16#27f#, 16#5b#, 16#77#, 16#35f#, 16#df#, 16#eff#, 16#1eff#, 16#bff#, 16#b7#, 16#3#, 16#13#, 16#1b#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (1, 2, 5, 9, 10, 10, 13, 13, 14, 9, 9, 12, 12, 14, 8, 9, 9, 12, 12, 14, 9, 10, 11, 9, 10, 10, 13, 13, 15, 9, 9, 12, 12, 14, 8, 8, 11, 11, 13, 8, 9, 9, 12, 12, 14, 8, 8, 11, 11, 13, 8, 9, 10, 9, 10, 10, 13, 13, 15, 9, 9, 12, 12, 14, 8, 8, 11, 11, 13, 8, 9, 9, 12, 12, 14, 8, 9, 9, 12, 12, 14, 10, 10, 11, 8, 9, 9, 12, 12, 14, 8, 8, 11, 11, 13, 8, 9, 9, 12, 12, 14, 7, 10, 10, 12, 7, 9, 10, 10, 13, 13, 14, 9, 5, 5, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 116
	);

	constant HYBRID_CODE_TABLE6_C  : hybrid_code_table_t := (
		input_code  => ("0^{004}       ", "0^{003}100    ", "0^{003}101    ", "0^{003}102    ", "0^{003}103    ", "0^{003}104    ", "0^{003}10X    ", "0^{003}11     ", "0^{003}12     ", "0^{003}13     ", "0^{003}14     ", "0^{003}1X     ", "0^{003}200    ", "0^{003}201    ", "0^{003}202    ", "0^{003}203    ", "0^{003}204    ", "0^{003}20X    ", "0^{003}21     ", "0^{003}22     ", "0^{003}23     ", "0^{003}24     ", "0^{003}2X     ", "0^{003}3      ", "0^{003}4      ", "0^{003}X      ", "0^{002}100    ", "0^{002}101    ", "0^{002}102    ", "0^{002}103    ", "0^{002}104    ", "0^{002}10X    ", "0^{002}11     ", "0^{002}12     ", "0^{002}13     ", "0^{002}14     ", "0^{002}1X     ", "0^{002}2      ", "0^{002}3      ", "0^{002}4      ", "0^{002}X      ", "0^{001}1      ", "0^{001}20     ", "0^{001}210    ", "0^{001}211    ", "0^{001}212    ", "0^{001}213    ", "0^{001}214    ", "0^{001}21X    ", "0^{001}22     ", "0^{001}23     ", "0^{001}24     ", "0^{001}2X     ", "0^{001}3      ", "0^{001}4      ", "0^{001}X      ", "0^{000}10     ", "0^{000}1100   ", "0^{000}1101   ", "0^{000}1102   ", "0^{000}1103   ", "0^{000}1104   ", "0^{000}110X   ", "0^{000}111    ", "0^{000}112    ", "0^{000}113    ", "0^{000}114    ", "0^{000}11X    ", "0^{000}12     ", "0^{000}13     ", "0^{000}14     ", "0^{000}1X     ", "0^{000}200    ", "0^{000}20100  ", "0^{000}20101  ", "0^{000}20102  ", "0^{000}20103  ", "0^{000}20104  ", "0^{000}2010X  ", "0^{000}2011   ", "0^{000}2012   ", "0^{000}2013   ", "0^{000}2014   ", "0^{000}201X   ", "0^{000}202    ", "0^{000}203    ", "0^{000}204    ", "0^{000}20X    ", "0^{000}21     ", "0^{000}22     ", "0^{000}23     ", "0^{000}24     ", "0^{000}2X     ", "0^{000}30     ", "0^{000}31     ", "0^{000}32     ", "0^{000}33     ", "0^{000}34     ", "0^{000}3X     ", "0^{000}4      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#2#, 16#13#, 16#4f#, 16#cf#, 16#6ff#, 16#eff#, 16#2fff#, 16#3b#, 16#7b#, 16#47f#, 16#27f#, 16#3ff#, 16#33#, 16#2f#, 16#af#, 16#1ff#, 16#9ff#, 16#1fff#, 16#7#, 16#47#, 16#67f#, 16#aff#, 16#13ff#, 16#5f#, 16#15f#, 16#7bf#, 16#15#, 16#27#, 16#67#, 16#17f#, 16#57f#, 16#bff#, 16#1d#, 16#3d#, 16#2bf#, 16#7f#, 16#1dff#, 16#6#, 16#57#, 16#d7#, 16#3f#, 16#0#, 16#e#, 16#3#, 16#37#, 16#b7#, 16#f7f#, 16#ff#, 16#17ff#, 16#d#, 16#23f#, 16#13f#, 16#37f#, 16#4b#, 16#2b#, 16#16f#, 16#4#, 16#23#, 16#77#, 16#f7#, 16#8ff#, 16#4ff#, 16#37ff#, 16#1b#, 16#5b#, 16#3bf#, 16#b7f#, 16#dff#, 16#9#, 16#ef#, 16#1ef#, 16#1bf#, 16#1#, 16#17#, 16#df#, 16#1df#, 16#1bff#, 16#7ff#, 16#7fff#, 16#f#, 16#8f#, 16#cff#, 16#2ff#, 16#fff#, 16#2d#, 16#33f#, 16#bf#, 16#77f#, 16#19#, 16#5#, 16#1f#, 16#11f#, 16#5bf#, 16#6b#, 16#9f#, 16#19f#, 16#5ff#, 16#15ff#, 16#3fff#, 16#b#, 16#6f#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (3, 6, 8, 8, 12, 12, 14, 7, 7, 11, 11, 13, 6, 8, 8, 12, 12, 14, 7, 7, 11, 12, 13, 9, 9, 11, 5, 7, 7, 11, 11, 13, 6, 6, 10, 11, 13, 4, 8, 8, 10, 3, 4, 6, 8, 8, 12, 12, 14, 6, 10, 10, 12, 7, 7, 9, 3, 6, 8, 8, 12, 12, 14, 7, 7, 11, 12, 13, 5, 9, 9, 11, 4, 7, 9, 9, 13, 13, 15, 8, 8, 12, 12, 14, 6, 10, 10, 12, 5, 5, 9, 9, 11, 7, 9, 9, 13, 13, 15, 7, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 101
	);

	constant HYBRID_CODE_TABLE7_C  : hybrid_code_table_t := (
		input_code  => ("0^{002}       ", "0^{001}100    ", "0^{001}1010   ", "0^{001}1011   ", "0^{001}1012   ", "0^{001}1013   ", "0^{001}1014   ", "0^{001}101X   ", "0^{001}1020   ", "0^{001}1021   ", "0^{001}1022   ", "0^{001}1023   ", "0^{001}1024   ", "0^{001}102X   ", "0^{001}103    ", "0^{001}104    ", "0^{001}10X    ", "0^{001}11     ", "0^{001}12     ", "0^{001}13     ", "0^{001}14     ", "0^{001}1X     ", "0^{001}20     ", "0^{001}21     ", "0^{001}220    ", "0^{001}221    ", "0^{001}222    ", "0^{001}223    ", "0^{001}224    ", "0^{001}22X    ", "0^{001}23     ", "0^{001}24     ", "0^{001}2X     ", "0^{001}3      ", "0^{001}4      ", "0^{001}X      ", "0^{000}1000   ", "0^{000}10010  ", "0^{000}10011  ", "0^{000}10012  ", "0^{000}10013  ", "0^{000}10014  ", "0^{000}1001X  ", "0^{000}10020  ", "0^{000}10021  ", "0^{000}10022  ", "0^{000}10023  ", "0^{000}10024  ", "0^{000}1002X  ", "0^{000}1003   ", "0^{000}1004   ", "0^{000}100X   ", "0^{000}101    ", "0^{000}102    ", "0^{000}103    ", "0^{000}104    ", "0^{000}10X    ", "0^{000}110    ", "0^{000}111    ", "0^{000}1120   ", "0^{000}1121   ", "0^{000}1122   ", "0^{000}1123   ", "0^{000}1124   ", "0^{000}112X   ", "0^{000}113    ", "0^{000}114    ", "0^{000}11X    ", "0^{000}12     ", "0^{000}13     ", "0^{000}14     ", "0^{000}1X     ", "0^{000}2      ", "0^{000}30     ", "0^{000}31     ", "0^{000}32     ", "0^{000}33     ", "0^{000}34     ", "0^{000}3X     ", "0^{000}4      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#d#, 16#77#, 16#15f#, 16#df#, 16#17ff#, 16#57ff#, 16#7fff#, 16#f#, 16#1df#, 16#3bf#, 16#37ff#, 16#77ff#, 16#17fff#, 16#1ff#, 16#9ff#, 16#3bff#, 16#2b#, 16#1b#, 16#4ff#, 16#cff#, 16#13ff#, 16#5#, 16#3b#, 16#37#, 16#9f#, 16#19f#, 16#27ff#, 16#67ff#, 16#5fff#, 16#2ff#, 16#aff#, 16#33ff#, 16#1ef#, 16#1f#, 16#27f#, 16#3#, 16#4f#, 16#3f#, 16#13f#, 16#fff#, 16#4fff#, 16#3fff#, 16#2f#, 16#bf#, 16#7f#, 16#2fff#, 16#6fff#, 16#ffff#, 16#5ff#, 16#dff#, 16#7ff#, 16#7#, 16#27#, 16#6ff#, 16#eff#, 16#bff#, 16#17#, 16#6f#, 16#5f#, 16#77f#, 16#ff#, 16#dfff#, 16#1bfff#, 16#3ffff#, 16#2bff#, 16#1bff#, 16#9fff#, 16#b#, 16#67f#, 16#17f#, 16#3ff#, 16#1#, 16#11f#, 16#57f#, 16#37f#, 16#1fff#, 16#bfff#, 16#1ffff#, 16#ef#, 16#1bf#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (1, 4, 7, 9, 9, 15, 15, 17, 7, 9, 10, 15, 15, 17, 12, 12, 14, 6, 6, 12, 12, 14, 4, 6, 7, 9, 9, 15, 15, 16, 12, 12, 14, 9, 9, 11, 4, 7, 9, 9, 15, 15, 16, 7, 9, 10, 15, 15, 17, 12, 12, 14, 6, 6, 12, 12, 14, 6, 8, 9, 11, 11, 16, 17, 18, 14, 14, 16, 6, 11, 11, 13, 3, 9, 11, 11, 16, 17, 18, 9, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 81
	);
	
	constant HYBRID_CODE_TABLE8_C  : hybrid_code_table_t := (
		input_code  => ("0^{010}       ", "0^{009}1      ", "0^{009}200    ", "0^{009}201    ", "0^{009}202    ", "0^{009}20X    ", "0^{009}21     ", "0^{009}22     ", "0^{009}2X     ", "0^{009}X      ", "0^{008}1      ", "0^{008}2      ", "0^{008}X      ", "0^{007}100    ", "0^{007}101    ", "0^{007}102    ", "0^{007}10X    ", "0^{007}11     ", "0^{007}12     ", "0^{007}1X     ", "0^{007}20     ", "0^{007}21     ", "0^{007}22     ", "0^{007}2X     ", "0^{007}X      ", "0^{006}1      ", "0^{006}200    ", "0^{006}201    ", "0^{006}202    ", "0^{006}20X    ", "0^{006}21     ", "0^{006}22     ", "0^{006}2X     ", "0^{006}X      ", "0^{005}1      ", "0^{005}2      ", "0^{005}X      ", "0^{004}10     ", "0^{004}11     ", "0^{004}12     ", "0^{004}1X     ", "0^{004}20     ", "0^{004}21     ", "0^{004}22     ", "0^{004}2X     ", "0^{004}X      ", "0^{003}1      ", "0^{003}200    ", "0^{003}201    ", "0^{003}202    ", "0^{003}20X    ", "0^{003}210    ", "0^{003}211    ", "0^{003}212    ", "0^{003}21X    ", "0^{003}22     ", "0^{003}2X     ", "0^{003}X      ", "0^{002}1      ", "0^{002}2      ", "0^{002}X      ", "0^{001}10     ", "0^{001}11     ", "0^{001}12     ", "0^{001}1X     ", "0^{001}2      ", "0^{001}X      ", "0^{000}100    ", "0^{000}101    ", "0^{000}102    ", "0^{000}10X    ", "0^{000}110    ", "0^{000}111    ", "0^{000}112    ", "0^{000}11X    ", "0^{000}12     ", "0^{000}1X     ", "0^{000}200    ", "0^{000}201    ", "0^{000}202    ", "0^{000}20X    ", "0^{000}210    ", "0^{000}211    ", "0^{000}212    ", "0^{000}21X    ", "0^{000}22     ", "0^{000}2X     ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#b#, 16#37#, 16#27f#, 16#17f#, 16#3fff#, 16#3bf#, 16#7f#, 16#5fff#, 16#dff#, 16#3#, 16#23#, 16#6ff#, 16#2b#, 16#df#, 16#1df#, 16#1fff#, 16#1f#, 16#11f#, 16#37ff#, 16#13#, 16#9f#, 16#19f#, 16#6fff#, 16#2ff#, 16#1d#, 16#33#, 16#5f#, 16#15f#, 16#fff#, 16#ef#, 16#1ef#, 16#17ff#, 16#4ff#, 16#9#, 16#19#, 16#37f#, 16#5#, 16#f7#, 16#f#, 16#1bff#, 16#15#, 16#8f#, 16#4f#, 16#7ff#, 16#1bf#, 16#1#, 16#d#, 16#cf#, 16#2f#, 16#27ff#, 16#af#, 16#77f#, 16#ff#, 16#ffff#, 16#77#, 16#bff#, 16#2bf#, 16#c#, 16#2#, 16#3f#, 16#a#, 16#3b#, 16#7b#, 16#3ff#, 16#4#, 16#16f#, 16#6#, 16#7#, 16#47#, 16#5ff#, 16#27#, 16#23f#, 16#13f#, 16#2fff#, 16#1b#, 16#1ff#, 16#e#, 16#67#, 16#17#, 16#13ff#, 16#57#, 16#33f#, 16#bf#, 16#7fff#, 16#5b#, 16#9ff#, 16#6f#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (3, 6, 7, 10, 10, 15, 10, 10, 15, 12, 6, 6, 11, 6, 9, 9, 15, 9, 9, 14, 6, 9, 9, 15, 11, 5, 6, 9, 9, 14, 9, 9, 14, 11, 5, 5, 11, 5, 8, 8, 13, 5, 8, 8, 14, 10, 4, 5, 8, 8, 14, 8, 11, 11, 16, 8, 13, 10, 4, 4, 10, 4, 7, 7, 13, 4, 9, 4, 7, 7, 12, 7, 10, 10, 15, 7, 12, 4, 7, 7, 13, 7, 10, 10, 16, 7, 12, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 88
	);

	constant HYBRID_CODE_TABLE9_C  : hybrid_code_table_t := (
		input_code  => ("0^{011}       ", "0^{010}10     ", "0^{010}11     ", "0^{010}12     ", "0^{010}1X     ", "0^{010}2      ", "0^{010}X      ", "0^{009}100    ", "0^{009}101    ", "0^{009}102    ", "0^{009}10X    ", "0^{009}11     ", "0^{009}12     ", "0^{009}1X     ", "0^{009}2      ", "0^{009}X      ", "0^{008}1000   ", "0^{008}1001   ", "0^{008}1002   ", "0^{008}100X   ", "0^{008}101    ", "0^{008}102    ", "0^{008}10X    ", "0^{008}11     ", "0^{008}12     ", "0^{008}1X     ", "0^{008}20     ", "0^{008}21     ", "0^{008}22     ", "0^{008}2X     ", "0^{008}X      ", "0^{007}1      ", "0^{007}200    ", "0^{007}201    ", "0^{007}202    ", "0^{007}20X    ", "0^{007}21     ", "0^{007}22     ", "0^{007}2X     ", "0^{007}X      ", "0^{006}1      ", "0^{006}2000   ", "0^{006}2001   ", "0^{006}2002   ", "0^{006}200X   ", "0^{006}201    ", "0^{006}202    ", "0^{006}20X    ", "0^{006}21     ", "0^{006}22     ", "0^{006}2X     ", "0^{006}X      ", "0^{005}1      ", "0^{005}2      ", "0^{005}X      ", "0^{004}10     ", "0^{004}11     ", "0^{004}12     ", "0^{004}1X     ", "0^{004}2      ", "0^{004}X      ", "0^{003}100    ", "0^{003}101    ", "0^{003}102    ", "0^{003}10X    ", "0^{003}11     ", "0^{003}12     ", "0^{003}1X     ", "0^{003}20     ", "0^{003}21     ", "0^{003}22     ", "0^{003}2X     ", "0^{003}X      ", "0^{002}1000   ", "0^{002}1001   ", "0^{002}1002   ", "0^{002}100X   ", "0^{002}101    ", "0^{002}102    ", "0^{002}10X    ", "0^{002}11     ", "0^{002}12     ", "0^{002}1X     ", "0^{002}200    ", "0^{002}201    ", "0^{002}202    ", "0^{002}20X    ", "0^{002}21     ", "0^{002}22     ", "0^{002}2X     ", "0^{002}X      ", "0^{001}1      ", "0^{001}2000   ", "0^{001}2001   ", "0^{001}2002   ", "0^{001}200X   ", "0^{001}201    ", "0^{001}202    ", "0^{001}20X    ", "0^{001}21     ", "0^{001}22     ", "0^{001}2X     ", "0^{001}X      ", "0^{000}1      ", "0^{000}2      ", "0^{000}X      ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#3b#, 16#1bf#, 16#3bf#, 16#17fff#, 16#1b#, 16#bff#, 16#7#, 16#7f#, 16#27f#, 16#ffff#, 16#33f#, 16#bf#, 16#3fff#, 16#13#, 16#3ff#, 16#27#, 16#17f#, 16#37f#, 16#1ffff#, 16#19f#, 16#2bf#, 16#bfff#, 16#9f#, 16#df#, 16#9fff#, 16#33#, 16#2df#, 16#1df#, 16#7fff#, 16#dff#, 16#3#, 16#b#, 16#3df#, 16#3f#, 16#5fff#, 16#1f#, 16#15f#, 16#efff#, 16#5ff#, 16#1d#, 16#2b#, 16#23f#, 16#13f#, 16#dfff#, 16#11f#, 16#35f#, 16#1fff#, 16#1ef#, 16#25f#, 16#6fff#, 16#9ff#, 16#9#, 16#19#, 16#1ff#, 16#5#, 16#12f#, 16#af#, 16#77ff#, 16#e#, 16#6ff#, 16#15#, 16#1af#, 16#6f#, 16#fff#, 16#b7#, 16#10f#, 16#17ff#, 16#1e#, 16#8f#, 16#18f#, 16#2fff#, 16#2ff#, 16#d#, 16#16f#, 16#ef#, 16#4fff#, 16#77#, 16#4f#, 16#57ff#, 16#97#, 16#57#, 16#47ff#, 16#1#, 16#14f#, 16#cf#, 16#afff#, 16#d7#, 16#1f7#, 16#27ff#, 16#4ff#, 16#6#, 16#11#, 16#1cf#, 16#2f#, 16#37ff#, 16#37#, 16#f#, 16#67ff#, 16#17#, 16#f7#, 16#7ff#, 16#ff#, 16#2#, 16#a#, 16#5f#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (2, 6, 10, 10, 17, 6, 12, 6, 10, 10, 17, 10, 10, 16, 6, 12, 6, 10, 10, 17, 9, 10, 16, 9, 10, 16, 6, 10, 10, 17, 12, 5, 6, 10, 10, 16, 9, 10, 16, 12, 5, 6, 10, 10, 16, 9, 10, 16, 9, 10, 16, 12, 5, 5, 12, 5, 9, 9, 15, 5, 11, 5, 9, 9, 15, 8, 9, 15, 5, 9, 9, 16, 11, 5, 9, 9, 15, 8, 9, 15, 8, 8, 15, 5, 9, 9, 16, 8, 9, 15, 11, 4, 5, 9, 9, 15, 8, 9, 15, 8, 9, 15, 11, 4, 4, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 106
	);
	
	constant HYBRID_CODE_TABLE10_C : hybrid_code_table_t := (
		input_code  => ("0^{009}       ", "0^{003}1      ", "0^{004}1      ", "0^{005}1      ", "0^{006}1      ", "0^{000}2      ", "0^{001}2      ", "0^{002}2      ", "0^{003}2      ", "0^{000}X      ", "0^{001}X      ", "0^{002}X      ", "0^{003}X      ", "0^{004}X      ", "0^{005}X      ", "0^{006}X      ", "0^{007}X      ", "0^{008}X      ", "0^{002}10     ", "0^{000}11     ", "0^{001}11     ", "0^{002}11     ", "0^{007}11     ", "0^{008}11     ", "0^{000}12     ", "0^{001}12     ", "0^{002}12     ", "0^{007}12     ", "0^{008}12     ", "0^{000}1X     ", "0^{001}1X     ", "0^{002}1X     ", "0^{007}1X     ", "0^{008}1X     ", "0^{008}20     ", "0^{004}21     ", "0^{005}21     ", "0^{006}21     ", "0^{007}21     ", "0^{008}21     ", "0^{004}22     ", "0^{005}22     ", "0^{006}22     ", "0^{007}22     ", "0^{008}22     ", "0^{004}2X     ", "0^{005}2X     ", "0^{006}2X     ", "0^{007}2X     ", "0^{008}2X     ", "0^{001}100    ", "0^{000}101    ", "0^{001}101    ", "0^{007}101    ", "0^{008}101    ", "0^{000}102    ", "0^{001}102    ", "0^{007}102    ", "0^{008}102    ", "0^{000}10X    ", "0^{001}10X    ", "0^{007}10X    ", "0^{008}10X    ", "0^{006}200    ", "0^{007}200    ", "0^{004}201    ", "0^{005}201    ", "0^{006}201    ", "0^{007}201    ", "0^{004}202    ", "0^{005}202    ", "0^{006}202    ", "0^{007}202    ", "0^{004}20X    ", "0^{005}20X    ", "0^{006}20X    ", "0^{007}20X    ", "0^{000}1000   ", "0^{008}1000   ", "0^{000}1001   ", "0^{007}1001   ", "0^{008}1001   ", "0^{000}1002   ", "0^{007}1002   ", "0^{008}1002   ", "0^{000}100X   ", "0^{007}100X   ", "0^{008}100X   ", "0^{005}2000   ", "0^{004}2001   ", "0^{005}2001   ", "0^{004}2002   ", "0^{005}2002   ", "0^{004}200X   ", "0^{005}200X   ", "0^{007}10000  ", "0^{007}10001  ", "0^{007}10002  ", "0^{007}1000X  ", "0^{004}20000  ", "0^{004}20001  ", "0^{004}20002  ", "0^{004}2000X  ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#19#, 16#18#, 16#19#, 16#1A#, 16#10#, 16#11#, 16#12#, 16#5#, 16#FFA#, 16#FFB#, 16#1FF8#, 16#1FF9#, 16#1FFA#, 16#1FFB#, 16#1FFC#, 16#1FFD#, 16#1FFE#, 16#15#, 16#1E8#, 16#1E9#, 16#26F#, 16#5F#, 16#1DF#, 16#3D6#, 16#3D7#, 16#16F#, 16#25F#, 16#3DF#, 16#1FFF#, 16#1FFF1#, 16#1FFF3#, 16#33FFF#, 16#7FFF#, 16#27#, 16#3DF#, 16#3E1#, 16#29F#, 16#15F#, 16#3F#, 16#3E0#, 16#3E2#, 16#7F#, 16#67F#, 16#77F#, 16#DFFF#, 16#3FFEE#, 16#3FFF0#, 16#BFFF#, 16#27FFF#, 16#D#, 16#AF#, 16#36F#, 16#23F#, 16#BF#, 16#6F#, 16#EF#, 16#13F#, 16#4FF#, 16#1FFF2#, 16#1FFF4#, 16#3FFFA#, 16#3FFFC#, 16#1B#, 16#17#, 16#31F#, 16#19F#, 16#35F#, 16#33F#, 16#9F#, 16#47F#, 16#17F#, 16#FF#, 16#3FFF7#, 16#3FFF9#, 16#2BFFF#, 16#37FFF#, 16#1D#, 16#37#, 16#2EF#, 16#3F5#, 16#3F6#, 16#1EF#, 16#7FA#, 16#7FB#, 16#15FFF#, 16#3FFFD#, 16#3FFFE#, 16#3B#, 16#39F#, 16#DF#, 16#27F#, 16#57F#, 16#13FFF#, 16#1BFFF#, 16#F#, 16#3BF#, 16#1FF#, 16#3FFFF#, 16#7#, 16#2DF#, 16#37F#, 16#3BFFF#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (1, 5, 5, 5, 5, 5, 5, 5, 5, 12, 12, 13, 13, 13, 13, 13, 13, 13, 5, 9, 9, 10, 10, 10, 10, 10, 10, 10, 10, 17, 17, 17, 18, 18, 6, 10, 10, 10, 10, 10, 10, 10, 11, 11, 11, 17, 18, 18, 18, 18, 5, 9, 10, 10, 10, 10, 10, 10, 11, 17, 17, 18, 18, 6, 6, 10, 10, 10, 10, 10, 11, 11, 11, 18, 18, 18, 18, 5, 6, 10, 10, 10, 10, 11, 11, 17, 18, 18, 6, 10, 10, 11, 11, 18, 18, 6, 10, 11, 18, 6, 10, 11, 18, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 103
	);

	constant HYBRID_CODE_TABLE11_C : hybrid_code_table_t := (
		input_code  => ("0^{016}       ", "0^{000}1      ", "0^{007}1      ", "0^{008}1      ", "0^{009}1      ", "0^{010}1      ", "0^{011}1      ", "0^{012}1      ", "0^{013}1      ", "0^{014}1      ", "0^{015}1      ", "0^{002}2      ", "0^{003}2      ", "0^{004}2      ", "0^{005}2      ", "0^{006}2      ", "0^{007}2      ", "0^{008}2      ", "0^{009}2      ", "0^{010}2      ", "0^{014}2      ", "0^{015}2      ", "0^{000}X      ", "0^{001}X      ", "0^{002}X      ", "0^{003}X      ", "0^{004}X      ", "0^{005}X      ", "0^{006}X      ", "0^{007}X      ", "0^{008}X      ", "0^{009}X      ", "0^{010}X      ", "0^{011}X      ", "0^{012}X      ", "0^{013}X      ", "0^{014}X      ", "0^{015}X      ", "0^{006}10     ", "0^{001}11     ", "0^{002}11     ", "0^{003}11     ", "0^{004}11     ", "0^{005}11     ", "0^{006}11     ", "0^{001}12     ", "0^{002}12     ", "0^{003}12     ", "0^{004}12     ", "0^{005}12     ", "0^{006}12     ", "0^{001}1X     ", "0^{002}1X     ", "0^{003}1X     ", "0^{004}1X     ", "0^{005}1X     ", "0^{006}1X     ", "0^{001}20     ", "0^{012}20     ", "0^{013}20     ", "0^{000}21     ", "0^{001}21     ", "0^{011}21     ", "0^{012}21     ", "0^{013}21     ", "0^{000}22     ", "0^{001}22     ", "0^{011}22     ", "0^{012}22     ", "0^{013}22     ", "0^{000}2X     ", "0^{001}2X     ", "0^{011}2X     ", "0^{012}2X     ", "0^{013}2X     ", "0^{004}100    ", "0^{005}100    ", "0^{001}101    ", "0^{002}101    ", "0^{003}101    ", "0^{004}101    ", "0^{005}101    ", "0^{001}102    ", "0^{002}102    ", "0^{003}102    ", "0^{004}102    ", "0^{005}102    ", "0^{001}10X    ", "0^{002}10X    ", "0^{003}10X    ", "0^{004}10X    ", "0^{005}10X    ", "0^{000}200    ", "0^{000}201    ", "0^{011}201    ", "0^{000}202    ", "0^{011}202    ", "0^{000}20X    ", "0^{011}20X    ", "0^{003}1000   ", "0^{001}1001   ", "0^{002}1001   ", "0^{003}1001   ", "0^{001}1002   ", "0^{002}1002   ", "0^{003}1002   ", "0^{001}100X   ", "0^{002}100X   ", "0^{003}100X   ", "0^{011}2000   ", "0^{011}2001   ", "0^{011}2002   ", "0^{011}200X   ", "0^{002}10000  ", "0^{001}10001  ", "0^{002}10001  ", "0^{001}10002  ", "0^{002}10002  ", "0^{001}1000X  ", "0^{002}1000X  ", "0^{001}100001 ", "0^{001}100002 ", "0^{001}10000X ", "0^{001}1000000", "0^{001}1000001", "0^{001}1000002", "0^{001}100000X", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#1#, 16#D#, 16#31#, 16#33#, 16#35#, 16#37#, 16#38#, 16#39#, 16#3A#, 16#3B#, 16#11#, 16#25#, 16#26#, 16#27#, 16#28#, 16#2D#, 16#32#, 16#34#, 16#36#, 16#4F#, 16#1F#, 16#3FF4#, 16#3FF5#, 16#3FF6#, 16#3FF7#, 16#3FF8#, 16#3FF9#, 16#3FFA#, 16#7FF6#, 16#7FF7#, 16#7FF8#, 16#7FF9#, 16#7FFA#, 16#7FFB#, 16#7FFC#, 16#7FFD#, 16#7FFE#, 16#1D#, 16#25F#, 16#DF#, 16#1DF#, 16#23F#, 16#2BF#, 16#17F#, 16#65F#, 16#4DF#, 16#5DF#, 16#63F#, 16#6BF#, 16#57F#, 16#7FFF#, 16#7FFF1#, 16#7FFF2#, 16#F7FFF#, 16#CFFFF#, 16#1FFFF#, 16#31#, 16#78#, 16#7A#, 16#5F#, 16#15F#, 16#FF3#, 16#FF5#, 16#9FF#, 16#45F#, 16#55F#, 16#FF4#, 16#FF6#, 16#5FF#, 16#FFFEC#, 16#FFFED#, 16#DFFFF#, 16#FFFFC#, 16#FFFFE#, 16#25#, 16#3D#, 16#2DF#, 16#3DF#, 16#13F#, 16#1BF#, 16#37F#, 16#6DF#, 16#7DF#, 16#53F#, 16#5BF#, 16#77F#, 16#7FFF2#, 16#7FFF4#, 16#FFFF#, 16#2FFFF#, 16#9FFFF#, 16#9#, 16#35F#, 16#EFF#, 16#75F#, 16#1FF#, 16#77FFF#, 16#BFFFF#, 16#15#, 16#3F#, 16#33F#, 16#3BF#, 16#43F#, 16#73F#, 16#7BF#, 16#57FFF#, 16#8FFFF#, 16#AFFFF#, 16#6F#, 16#DFF#, 16#3FF#, 16#FFFFF#, 16#35#, 16#BF#, 16#7F#, 16#4BF#, 16#47F#, 16#4FFFF#, 16#6FFFF#, 16#27F#, 16#67F#, 16#EFFFF#, 16#3#, 16#FF#, 16#4FF#, 16#5FFFF#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (1, 5, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 14, 14, 14, 14, 14, 14, 14, 15, 15, 15, 15, 15, 15, 15, 15, 15, 6, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 19, 19, 19, 20, 20, 20, 6, 7, 7, 11, 11, 12, 12, 12, 11, 11, 12, 12, 12, 20, 20, 20, 20, 20, 6, 6, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 19, 19, 20, 20, 20, 6, 11, 12, 11, 12, 20, 20, 6, 11, 11, 11, 11, 11, 11, 19, 20, 20, 7, 12, 12, 20, 6, 11, 11, 11, 11, 20, 20, 11, 11, 20, 6, 11, 12, 20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 127
	);

	constant HYBRID_CODE_TABLE12_C : hybrid_code_table_t := (
		input_code  => ("0^{027}       ", "0^{000}1      ", "0^{001}1      ", "0^{002}1      ", "0^{003}1      ", "0^{004}1      ", "0^{005}1      ", "0^{006}1      ", "0^{007}1      ", "0^{008}1      ", "0^{011}1      ", "0^{012}1      ", "0^{013}1      ", "0^{014}1      ", "0^{015}1      ", "0^{016}1      ", "0^{017}1      ", "0^{018}1      ", "0^{019}1      ", "0^{020}1      ", "0^{021}1      ", "0^{022}1      ", "0^{023}1      ", "0^{024}1      ", "0^{025}1      ", "0^{026}1      ", "0^{000}2      ", "0^{004}2      ", "0^{005}2      ", "0^{006}2      ", "0^{007}2      ", "0^{011}2      ", "0^{012}2      ", "0^{013}2      ", "0^{014}2      ", "0^{015}2      ", "0^{016}2      ", "0^{017}2      ", "0^{018}2      ", "0^{019}2      ", "0^{020}2      ", "0^{021}2      ", "0^{022}2      ", "0^{023}2      ", "0^{024}2      ", "0^{025}2      ", "0^{026}2      ", "0^{000}X      ", "0^{001}X      ", "0^{002}X      ", "0^{003}X      ", "0^{004}X      ", "0^{005}X      ", "0^{006}X      ", "0^{007}X      ", "0^{008}X      ", "0^{009}X      ", "0^{010}X      ", "0^{011}X      ", "0^{012}X      ", "0^{013}X      ", "0^{014}X      ", "0^{015}X      ", "0^{016}X      ", "0^{017}X      ", "0^{018}X      ", "0^{019}X      ", "0^{020}X      ", "0^{021}X      ", "0^{022}X      ", "0^{023}X      ", "0^{024}X      ", "0^{025}X      ", "0^{026}X      ", "0^{010}10     ", "0^{009}11     ", "0^{010}11     ", "0^{009}12     ", "0^{010}12     ", "0^{009}1X     ", "0^{010}1X     ", "0^{003}20     ", "0^{001}21     ", "0^{002}21     ", "0^{003}21     ", "0^{001}22     ", "0^{002}22     ", "0^{003}22     ", "0^{001}2X     ", "0^{002}2X     ", "0^{003}2X     ", "0^{009}100    ", "0^{009}101    ", "0^{009}102    ", "0^{009}10X    ", "0^{002}200    ", "0^{001}201    ", "0^{002}201    ", "0^{001}202    ", "0^{002}202    ", "0^{001}20X    ", "0^{002}20X    ", "0^{001}2000   ", "0^{001}2001   ", "0^{001}2002   ", "0^{001}200X   ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#1#, 16#22#, 16#23#, 16#24#, 16#25#, 16#26#, 16#27#, 16#28#, 16#29#, 16#3D#, 16#62#, 16#64#, 16#66#, 16#68#, 16#6A#, 16#6C#, 16#6E#, 16#70#, 16#72#, 16#74#, 16#76#, 16#78#, 16#7A#, 16#7C#, 16#7E#, 16#21#, 16#15#, 16#58#, 16#59#, 16#5A#, 16#7D#, 16#63#, 16#65#, 16#67#, 16#69#, 16#71#, 16#73#, 16#75#, 16#77#, 16#79#, 16#7B#, 16#7D#, 16#7F#, 16#81#, 16#83#, 16#7F#, 16#7FEC#, 16#7FED#, 16#7FEE#, 16#7FEF#, 16#7FF0#, 16#7FF1#, 16#7FF2#, 16#7FF3#, 16#7FF4#, 16#7FF5#, 16#7FF6#, 16#7FF7#, 16#7FF8#, 16#FFF2#, 16#FFF3#, 16#FFF4#, 16#FFF5#, 16#FFF6#, 16#FFF7#, 16#FFF8#, 16#FFF9#, 16#FFFA#, 16#FFFB#, 16#FFFC#, 16#FFFD#, 16#FFFE#, 16#FFFF#, 16#3#, 16#FF6#, 16#FF7#, 16#1FF8#, 16#1FF9#, 16#FFFFC#, 16#FFFFD#, 16#55#, 16#FF#, 16#FF1#, 16#FF3#, 16#9FF#, 16#1FF3#, 16#1FF5#, 16#1FFFF#, 16#FFFF9#, 16#FFFFB#, 16#43#, 16#1FF#, 16#BFF#, 16#7FFFF#, 16#35#, 16#FF2#, 16#FF4#, 16#1FF4#, 16#1FF6#, 16#5FFFF#, 16#FFFFF#, 16#75#, 16#AFF#, 16#1DFF#, 16#1FFFFF#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (1, 6, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 17, 7, 12, 12, 13, 13, 20, 20, 7, 12, 12, 12, 13, 13, 13, 20, 20, 20, 7, 12, 13, 20, 7, 12, 12, 13, 13, 20, 21, 7, 12, 13, 21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 106
	);

	constant HYBRID_CODE_TABLE13_C : hybrid_code_table_t := (
		input_code  => ("0^{046}       ", "0^{001}1      ", "0^{002}1      ", "0^{003}1      ", "0^{004}1      ", "0^{005}1      ", "0^{006}1      ", "0^{007}1      ", "0^{008}1      ", "0^{009}1      ", "0^{010}1      ", "0^{011}1      ", "0^{012}1      ", "0^{013}1      ", "0^{014}1      ", "0^{015}1      ", "0^{016}1      ", "0^{017}1      ", "0^{018}1      ", "0^{019}1      ", "0^{020}1      ", "0^{021}1      ", "0^{022}1      ", "0^{023}1      ", "0^{024}1      ", "0^{025}1      ", "0^{026}1      ", "0^{027}1      ", "0^{028}1      ", "0^{029}1      ", "0^{030}1      ", "0^{031}1      ", "0^{032}1      ", "0^{033}1      ", "0^{034}1      ", "0^{035}1      ", "0^{036}1      ", "0^{037}1      ", "0^{038}1      ", "0^{039}1      ", "0^{040}1      ", "0^{041}1      ", "0^{042}1      ", "0^{043}1      ", "0^{044}1      ", "0^{045}1      ", "0^{000}2      ", "0^{001}2      ", "0^{002}2      ", "0^{003}2      ", "0^{004}2      ", "0^{005}2      ", "0^{006}2      ", "0^{007}2      ", "0^{008}2      ", "0^{009}2      ", "0^{010}2      ", "0^{011}2      ", "0^{012}2      ", "0^{013}2      ", "0^{014}2      ", "0^{015}2      ", "0^{016}2      ", "0^{017}2      ", "0^{018}2      ", "0^{019}2      ", "0^{020}2      ", "0^{021}2      ", "0^{022}2      ", "0^{023}2      ", "0^{024}2      ", "0^{025}2      ", "0^{026}2      ", "0^{027}2      ", "0^{028}2      ", "0^{029}2      ", "0^{030}2      ", "0^{031}2      ", "0^{032}2      ", "0^{033}2      ", "0^{034}2      ", "0^{035}2      ", "0^{036}2      ", "0^{037}2      ", "0^{038}2      ", "0^{039}2      ", "0^{040}2      ", "0^{041}2      ", "0^{042}2      ", "0^{043}2      ", "0^{044}2      ", "0^{045}2      ", "0^{000}X      ", "0^{001}X      ", "0^{002}X      ", "0^{003}X      ", "0^{004}X      ", "0^{005}X      ", "0^{006}X      ", "0^{007}X      ", "0^{008}X      ", "0^{009}X      ", "0^{010}X      ", "0^{011}X      ", "0^{012}X      ", "0^{013}X      ", "0^{014}X      ", "0^{015}X      ", "0^{016}X      ", "0^{017}X      ", "0^{018}X      ", "0^{019}X      ", "0^{020}X      ", "0^{021}X      ", "0^{022}X      ", "0^{023}X      ", "0^{024}X      ", "0^{025}X      ", "0^{026}X      ", "0^{027}X      ", "0^{028}X      ", "0^{029}X      ", "0^{030}X      ", "0^{031}X      ", "0^{032}X      ", "0^{033}X      ", "0^{034}X      ", "0^{035}X      ", "0^{036}X      ", "0^{037}X      ", "0^{038}X      ", "0^{039}X      ", "0^{040}X      ", "0^{041}X      ", "0^{042}X      ", "0^{043}X      ", "0^{044}X      ", "0^{045}X      ", "0^{000}11     ", "0^{000}12     ", "0^{000}1X     ", "0^{000}100    ", "0^{000}101    ", "0^{000}102    ", "0^{000}10X    ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              ", "              "),
		output_code => (16#0#, 16#41#, 16#43#, 16#46#, 16#48#, 16#4A#, 16#4C#, 16#4E#, 16#50#, 16#52#, 16#53#, 16#54#, 16#55#, 16#56#, 16#57#, 16#58#, 16#59#, 16#5A#, 16#5B#, 16#5C#, 16#5D#, 16#5E#, 16#5F#, 16#60#, 16#61#, 16#62#, 16#63#, 16#DA#, 16#DC#, 16#DE#, 16#E0#, 16#E2#, 16#E4#, 16#E6#, 16#E8#, 16#EA#, 16#EC#, 16#EE#, 16#F0#, 16#F2#, 16#F4#, 16#F6#, 16#F8#, 16#FA#, 16#FC#, 16#FE#, 16#40#, 16#42#, 16#44#, 16#47#, 16#49#, 16#4B#, 16#4D#, 16#4F#, 16#51#, 16#C8#, 16#C9#, 16#CA#, 16#CB#, 16#CC#, 16#CD#, 16#CE#, 16#CF#, 16#D0#, 16#D1#, 16#D2#, 16#D3#, 16#D4#, 16#D5#, 16#D6#, 16#D7#, 16#D8#, 16#D9#, 16#DB#, 16#DD#, 16#DF#, 16#E1#, 16#E3#, 16#E5#, 16#E7#, 16#E9#, 16#EB#, 16#ED#, 16#EF#, 16#F1#, 16#F3#, 16#F5#, 16#F7#, 16#F9#, 16#FB#, 16#FD#, 16#FF#, 16#7FE0#, 16#7FE1#, 16#7FE2#, 16#7FE3#, 16#7FE4#, 16#7FE5#, 16#7FE6#, 16#7FE7#, 16#7FE8#, 16#7FE9#, 16#7FEA#, 16#7FEB#, 16#7FEC#, 16#7FED#, 16#7FEE#, 16#7FEF#, 16#7FF0#, 16#7FF1#, 16#FFE4#, 16#FFE5#, 16#FFE6#, 16#FFE7#, 16#FFE8#, 16#FFE9#, 16#FFEA#, 16#FFEB#, 16#FFEC#, 16#FFED#, 16#FFEE#, 16#FFEF#, 16#FFF0#, 16#FFF1#, 16#FFF2#, 16#FFF3#, 16#FFF4#, 16#FFF5#, 16#FFF6#, 16#FFF7#, 16#FFF8#, 16#FFF9#, 16#FFFA#, 16#FFFB#, 16#FFFC#, 16#FFFD#, 16#FFFE#, 16#FFFF#, 16#1FF#, 16#9FF#, 16#1FFFF#, 16#51#, 16#5FF#, 16#DFF#, 16#3FFFF#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#, 16#0#),
		output_size => (1, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 17, 12, 12, 18, 7, 12, 12, 18, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
		table_size	=> 145
	);
	
	constant HYBRID_CODE_TABLE14_C : hybrid_code_table_t := (
		input_code  => ("0^{085}       ", "0^{000}1      ", "0^{001}1      ", "0^{002}1      ", "0^{003}1      ", "0^{004}1      ", "0^{005}1      ", "0^{006}1      ", "0^{007}1      ", "0^{008}1      ", "0^{009}1      ", "0^{010}1      ", "0^{011}1      ", "0^{012}1      ", "0^{013}1      ", "0^{014}1      ", "0^{015}1      ", "0^{016}1      ", "0^{017}1      ", "0^{018}1      ", "0^{019}1      ", "0^{020}1      ", "0^{021}1      ", "0^{022}1      ", "0^{023}1      ", "0^{024}1      ", "0^{025}1      ", "0^{026}1      ", "0^{027}1      ", "0^{028}1      ", "0^{029}1      ", "0^{030}1      ", "0^{031}1      ", "0^{032}1      ", "0^{033}1      ", "0^{034}1      ", "0^{035}1      ", "0^{036}1      ", "0^{037}1      ", "0^{038}1      ", "0^{039}1      ", "0^{040}1      ", "0^{041}1      ", "0^{042}1      ", "0^{043}1      ", "0^{044}1      ", "0^{045}1      ", "0^{046}1      ", "0^{047}1      ", "0^{048}1      ", "0^{049}1      ", "0^{050}1      ", "0^{051}1      ", "0^{052}1      ", "0^{053}1      ", "0^{054}1      ", "0^{055}1      ", "0^{056}1      ", "0^{057}1      ", "0^{058}1      ", "0^{059}1      ", "0^{060}1      ", "0^{061}1      ", "0^{062}1      ", "0^{063}1      ", "0^{064}1      ", "0^{065}1      ", "0^{066}1      ", "0^{067}1      ", "0^{068}1      ", "0^{069}1      ", "0^{070}1      ", "0^{071}1      ", "0^{072}1      ", "0^{073}1      ", "0^{074}1      ", "0^{075}1      ", "0^{076}1      ", "0^{077}1      ", "0^{078}1      ", "0^{079}1      ", "0^{080}1      ", "0^{081}1      ", "0^{082}1      ", "0^{083}1      ", "0^{084}1      ", "0^{000}2      ", "0^{001}2      ", "0^{002}2      ", "0^{003}2      ", "0^{004}2      ", "0^{005}2      ", "0^{006}2      ", "0^{007}2      ", "0^{008}2      ", "0^{009}2      ", "0^{010}2      ", "0^{011}2      ", "0^{012}2      ", "0^{013}2      ", "0^{014}2      ", "0^{015}2      ", "0^{016}2      ", "0^{017}2      ", "0^{018}2      ", "0^{019}2      ", "0^{020}2      ", "0^{021}2      ", "0^{022}2      ", "0^{023}2      ", "0^{024}2      ", "0^{025}2      ", "0^{026}2      ", "0^{027}2      ", "0^{028}2      ", "0^{029}2      ", "0^{030}2      ", "0^{031}2      ", "0^{032}2      ", "0^{033}2      ", "0^{034}2      ", "0^{035}2      ", "0^{036}2      ", "0^{037}2      ", "0^{038}2      ", "0^{039}2      ", "0^{040}2      ", "0^{041}2      ", "0^{042}2      ", "0^{043}2      ", "0^{044}2      ", "0^{045}2      ", "0^{046}2      ", "0^{047}2      ", "0^{048}2      ", "0^{049}2      ", "0^{050}2      ", "0^{051}2      ", "0^{052}2      ", "0^{053}2      ", "0^{054}2      ", "0^{055}2      ", "0^{056}2      ", "0^{057}2      ", "0^{058}2      ", "0^{059}2      ", "0^{060}2      ", "0^{061}2      ", "0^{062}2      ", "0^{063}2      ", "0^{064}2      ", "0^{065}2      ", "0^{066}2      ", "0^{067}2      ", "0^{068}2      ", "0^{069}2      ", "0^{070}2      ", "0^{071}2      ", "0^{072}2      ", "0^{073}2      ", "0^{074}2      ", "0^{075}2      ", "0^{076}2      ", "0^{077}2      ", "0^{078}2      ", "0^{079}2      ", "0^{080}2      ", "0^{081}2      ", "0^{082}2      ", "0^{083}2      ", "0^{084}2      ", "0^{000}X      ", "0^{001}X      ", "0^{002}X      ", "0^{003}X      ", "0^{004}X      ", "0^{005}X      ", "0^{006}X      ", "0^{007}X      ", "0^{008}X      ", "0^{009}X      ", "0^{010}X      ", "0^{011}X      ", "0^{012}X      ", "0^{013}X      ", "0^{014}X      ", "0^{015}X      ", "0^{016}X      ", "0^{017}X      ", "0^{018}X      ", "0^{019}X      ", "0^{020}X      ", "0^{021}X      ", "0^{022}X      ", "0^{023}X      ", "0^{024}X      ", "0^{025}X      ", "0^{026}X      ", "0^{027}X      ", "0^{028}X      ", "0^{029}X      ", "0^{030}X      ", "0^{031}X      ", "0^{032}X      ", "0^{033}X      ", "0^{034}X      ", "0^{035}X      ", "0^{036}X      ", "0^{037}X      ", "0^{038}X      ", "0^{039}X      ", "0^{040}X      ", "0^{041}X      ", "0^{042}X      ", "0^{043}X      ", "0^{044}X      ", "0^{045}X      ", "0^{046}X      ", "0^{047}X      ", "0^{048}X      ", "0^{049}X      ", "0^{050}X      ", "0^{051}X      ", "0^{052}X      ", "0^{053}X      ", "0^{054}X      ", "0^{055}X      ", "0^{056}X      ", "0^{057}X      ", "0^{058}X      ", "0^{059}X      ", "0^{060}X      ", "0^{061}X      ", "0^{062}X      ", "0^{063}X      ", "0^{064}X      ", "0^{065}X      ", "0^{066}X      ", "0^{067}X      ", "0^{068}X      ", "0^{069}X      ", "0^{070}X      ", "0^{071}X      ", "0^{072}X      ", "0^{073}X      ", "0^{074}X      ", "0^{075}X      ", "0^{076}X      ", "0^{077}X      ", "0^{078}X      ", "0^{079}X      ", "0^{080}X      ", "0^{081}X      ", "0^{082}X      ", "0^{083}X      ", "0^{084}X      ", "              "),
		output_code => (16#0#, 16#80#, 16#82#, 16#84#, 16#86#, 16#88#, 16#8A#, 16#8C#, 16#8E#, 16#90#, 16#92#, 16#94#, 16#96#, 16#98#, 16#9A#, 16#9C#, 16#9E#, 16#A0#, 16#A2#, 16#A4#, 16#A6#, 16#A8#, 16#AA#, 16#AB#, 16#AC#, 16#AD#, 16#AE#, 16#AF#, 16#B0#, 16#B1#, 16#B2#, 16#B3#, 16#B4#, 16#B5#, 16#B6#, 16#B7#, 16#B8#, 16#B9#, 16#BA#, 16#BB#, 16#BC#, 16#BD#, 16#BE#, 16#BF#, 16#C0#, 16#C1#, 16#C2#, 16#C3#, 16#C4#, 16#C5#, 16#C6#, 16#C7#, 16#C8#, 16#C9#, 16#CA#, 16#CB#, 16#CC#, 16#CD#, 16#CE#, 16#CF#, 16#D0#, 16#D1#, 16#D2#, 16#D3#, 16#D4#, 16#D5#, 16#1D8#, 16#1DA#, 16#1DC#, 16#1DE#, 16#1E0#, 16#1E2#, 16#1E4#, 16#1E6#, 16#1E8#, 16#1EA#, 16#1EC#, 16#1EE#, 16#1F0#, 16#1F2#, 16#1F4#, 16#1F6#, 16#1F8#, 16#1FA#, 16#1FC#, 16#1FE#, 16#81#, 16#83#, 16#85#, 16#87#, 16#89#, 16#8B#, 16#8D#, 16#8F#, 16#91#, 16#93#, 16#95#, 16#97#, 16#99#, 16#9B#, 16#9D#, 16#9F#, 16#A1#, 16#A3#, 16#A5#, 16#A7#, 16#A9#, 16#1AC#, 16#1AD#, 16#1AE#, 16#1AF#, 16#1B0#, 16#1B1#, 16#1B2#, 16#1B3#, 16#1B4#, 16#1B5#, 16#1B6#, 16#1B7#, 16#1B8#, 16#1B9#, 16#1BA#, 16#1BB#, 16#1BC#, 16#1BD#, 16#1BE#, 16#1BF#, 16#1C0#, 16#1C1#, 16#1C2#, 16#1C3#, 16#1C4#, 16#1C5#, 16#1C6#, 16#1C7#, 16#1C8#, 16#1C9#, 16#1CA#, 16#1CB#, 16#1CC#, 16#1CD#, 16#1CE#, 16#1CF#, 16#1D0#, 16#1D1#, 16#1D2#, 16#1D3#, 16#1D4#, 16#1D5#, 16#1D6#, 16#1D7#, 16#1D9#, 16#1DB#, 16#1DD#, 16#1DF#, 16#1E1#, 16#1E3#, 16#1E5#, 16#1E7#, 16#1E9#, 16#1EB#, 16#1ED#, 16#1EF#, 16#1F1#, 16#1F3#, 16#1F5#, 16#1F7#, 16#1F9#, 16#1FB#, 16#1FD#, 16#1FF#, 16#FFC0#, 16#FFC1#, 16#FFC2#, 16#FFC3#, 16#FFC4#, 16#FFC5#, 16#FFC6#, 16#FFC7#, 16#FFC8#, 16#FFC9#, 16#FFCA#, 16#FFCB#, 16#FFCC#, 16#FFCD#, 16#FFCE#, 16#FFCF#, 16#FFD0#, 16#FFD1#, 16#FFD2#, 16#FFD3#, 16#FFD4#, 16#FFD5#, 16#FFD6#, 16#FFD7#, 16#FFD8#, 16#FFD9#, 16#FFDA#, 16#FFDB#, 16#FFDC#, 16#FFDD#, 16#FFDE#, 16#FFDF#, 16#FFE0#, 16#FFE1#, 16#FFE2#, 16#FFE3#, 16#FFE4#, 16#FFE5#, 16#FFE6#, 16#FFE7#, 16#FFE8#, 16#FFE9#, 16#FFEA#, 16#1FFD6#, 16#1FFD7#, 16#1FFD8#, 16#1FFD9#, 16#1FFDA#, 16#1FFDB#, 16#1FFDC#, 16#1FFDD#, 16#1FFDE#, 16#1FFDF#, 16#1FFE0#, 16#1FFE1#, 16#1FFE2#, 16#1FFE3#, 16#1FFE4#, 16#1FFE5#, 16#1FFE6#, 16#1FFE7#, 16#1FFE8#, 16#1FFE9#, 16#1FFEA#, 16#1FFEB#, 16#1FFEC#, 16#1FFED#, 16#1FFEE#, 16#1FFEF#, 16#1FFF0#, 16#1FFF1#, 16#1FFF2#, 16#1FFF3#, 16#1FFF4#, 16#1FFF5#, 16#1FFF6#, 16#1FFF7#, 16#1FFF8#, 16#1FFF9#, 16#1FFFA#, 16#1FFFB#, 16#1FFFC#, 16#1FFFD#, 16#1FFFE#, 16#1FFFF#, 16#0#),
		output_size => (1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 10, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 17, 0),
		table_size	=> 256
	);
	
	constant HYBRID_CODE_TABLE15_C : hybrid_code_table_t := (
		input_code  => ("0^{256}       ", "0^{000}X      ", "0^{001}X      ", "0^{002}X      ", "0^{003}X      ", "0^{004}X      ", "0^{005}X      ", "0^{006}X      ", "0^{007}X      ", "0^{008}X      ", "0^{009}X      ", "0^{010}X      ", "0^{011}X      ", "0^{012}X      ", "0^{013}X      ", "0^{014}X      ", "0^{015}X      ", "0^{016}X      ", "0^{017}X      ", "0^{018}X      ", "0^{019}X      ", "0^{020}X      ", "0^{021}X      ", "0^{022}X      ", "0^{023}X      ", "0^{024}X      ", "0^{025}X      ", "0^{026}X      ", "0^{027}X      ", "0^{028}X      ", "0^{029}X      ", "0^{030}X      ", "0^{031}X      ", "0^{032}X      ", "0^{033}X      ", "0^{034}X      ", "0^{035}X      ", "0^{036}X      ", "0^{037}X      ", "0^{038}X      ", "0^{039}X      ", "0^{040}X      ", "0^{041}X      ", "0^{042}X      ", "0^{043}X      ", "0^{044}X      ", "0^{045}X      ", "0^{046}X      ", "0^{047}X      ", "0^{048}X      ", "0^{049}X      ", "0^{050}X      ", "0^{051}X      ", "0^{052}X      ", "0^{053}X      ", "0^{054}X      ", "0^{055}X      ", "0^{056}X      ", "0^{057}X      ", "0^{058}X      ", "0^{059}X      ", "0^{060}X      ", "0^{061}X      ", "0^{062}X      ", "0^{063}X      ", "0^{064}X      ", "0^{065}X      ", "0^{066}X      ", "0^{067}X      ", "0^{068}X      ", "0^{069}X      ", "0^{070}X      ", "0^{071}X      ", "0^{072}X      ", "0^{073}X      ", "0^{074}X      ", "0^{075}X      ", "0^{076}X      ", "0^{077}X      ", "0^{078}X      ", "0^{079}X      ", "0^{080}X      ", "0^{081}X      ", "0^{082}X      ", "0^{083}X      ", "0^{084}X      ", "0^{085}X      ", "0^{086}X      ", "0^{087}X      ", "0^{088}X      ", "0^{089}X      ", "0^{090}X      ", "0^{091}X      ", "0^{092}X      ", "0^{093}X      ", "0^{094}X      ", "0^{095}X      ", "0^{096}X      ", "0^{097}X      ", "0^{098}X      ", "0^{099}X      ", "0^{100}X      ", "0^{101}X      ", "0^{102}X      ", "0^{103}X      ", "0^{104}X      ", "0^{105}X      ", "0^{106}X      ", "0^{107}X      ", "0^{108}X      ", "0^{109}X      ", "0^{110}X      ", "0^{111}X      ", "0^{112}X      ", "0^{113}X      ", "0^{114}X      ", "0^{115}X      ", "0^{116}X      ", "0^{117}X      ", "0^{118}X      ", "0^{119}X      ", "0^{120}X      ", "0^{121}X      ", "0^{122}X      ", "0^{123}X      ", "0^{124}X      ", "0^{125}X      ", "0^{126}X      ", "0^{127}X      ", "0^{128}X      ", "0^{129}X      ", "0^{130}X      ", "0^{131}X      ", "0^{132}X      ", "0^{133}X      ", "0^{134}X      ", "0^{135}X      ", "0^{136}X      ", "0^{137}X      ", "0^{138}X      ", "0^{139}X      ", "0^{140}X      ", "0^{141}X      ", "0^{142}X      ", "0^{143}X      ", "0^{144}X      ", "0^{145}X      ", "0^{146}X      ", "0^{147}X      ", "0^{148}X      ", "0^{149}X      ", "0^{150}X      ", "0^{151}X      ", "0^{152}X      ", "0^{153}X      ", "0^{154}X      ", "0^{155}X      ", "0^{156}X      ", "0^{157}X      ", "0^{158}X      ", "0^{159}X      ", "0^{160}X      ", "0^{161}X      ", "0^{162}X      ", "0^{163}X      ", "0^{164}X      ", "0^{165}X      ", "0^{166}X      ", "0^{167}X      ", "0^{168}X      ", "0^{169}X      ", "0^{170}X      ", "0^{171}X      ", "0^{172}X      ", "0^{173}X      ", "0^{174}X      ", "0^{175}X      ", "0^{176}X      ", "0^{177}X      ", "0^{178}X      ", "0^{179}X      ", "0^{180}X      ", "0^{181}X      ", "0^{182}X      ", "0^{183}X      ", "0^{184}X      ", "0^{185}X      ", "0^{186}X      ", "0^{187}X      ", "0^{188}X      ", "0^{189}X      ", "0^{190}X      ", "0^{191}X      ", "0^{192}X      ", "0^{193}X      ", "0^{194}X      ", "0^{195}X      ", "0^{196}X      ", "0^{197}X      ", "0^{198}X      ", "0^{199}X      ", "0^{200}X      ", "0^{201}X      ", "0^{202}X      ", "0^{203}X      ", "0^{204}X      ", "0^{205}X      ", "0^{206}X      ", "0^{207}X      ", "0^{208}X      ", "0^{209}X      ", "0^{210}X      ", "0^{211}X      ", "0^{212}X      ", "0^{213}X      ", "0^{214}X      ", "0^{215}X      ", "0^{216}X      ", "0^{217}X      ", "0^{218}X      ", "0^{219}X      ", "0^{220}X      ", "0^{221}X      ", "0^{222}X      ", "0^{223}X      ", "0^{224}X      ", "0^{225}X      ", "0^{226}X      ", "0^{227}X      ", "0^{228}X      ", "0^{229}X      ", "0^{230}X      ", "0^{231}X      ", "0^{232}X      ", "0^{233}X      ", "0^{234}X      ", "0^{235}X      ", "0^{236}X      ", "0^{237}X      ", "0^{238}X      ", "0^{239}X      ", "0^{240}X      ", "0^{241}X      ", "0^{242}X      ", "0^{243}X      ", "0^{244}X      ", "0^{245}X      ", "0^{246}X      ", "0^{247}X      ", "0^{248}X      ", "0^{249}X      ", "0^{250}X      ", "0^{251}X      ", "0^{252}X      ", "0^{253}X      ", "0^{254}X      ", "0^{255}X      "),
		output_code => (16#0#, 16#100#, 16#101#, 16#102#, 16#103#, 16#104#, 16#105#, 16#106#, 16#107#, 16#108#, 16#109#, 16#10A#, 16#10B#, 16#10C#, 16#10D#, 16#10E#, 16#10F#, 16#110#, 16#111#, 16#112#, 16#113#, 16#114#, 16#115#, 16#116#, 16#117#, 16#118#, 16#119#, 16#11A#, 16#11B#, 16#11C#, 16#11D#, 16#11E#, 16#11F#, 16#120#, 16#121#, 16#122#, 16#123#, 16#124#, 16#125#, 16#126#, 16#127#, 16#128#, 16#129#, 16#12A#, 16#12B#, 16#12C#, 16#12D#, 16#12E#, 16#12F#, 16#130#, 16#131#, 16#132#, 16#133#, 16#134#, 16#135#, 16#136#, 16#137#, 16#138#, 16#139#, 16#13A#, 16#13B#, 16#13C#, 16#13D#, 16#13E#, 16#13F#, 16#140#, 16#141#, 16#142#, 16#143#, 16#144#, 16#145#, 16#146#, 16#147#, 16#148#, 16#149#, 16#14A#, 16#14B#, 16#14C#, 16#14D#, 16#14E#, 16#14F#, 16#150#, 16#151#, 16#152#, 16#153#, 16#154#, 16#155#, 16#156#, 16#157#, 16#158#, 16#159#, 16#15A#, 16#15B#, 16#15C#, 16#15D#, 16#15E#, 16#15F#, 16#160#, 16#161#, 16#162#, 16#163#, 16#164#, 16#165#, 16#166#, 16#167#, 16#168#, 16#169#, 16#16A#, 16#16B#, 16#16C#, 16#16D#, 16#16E#, 16#16F#, 16#170#, 16#171#, 16#172#, 16#173#, 16#174#, 16#175#, 16#176#, 16#177#, 16#178#, 16#179#, 16#17A#, 16#17B#, 16#17C#, 16#17D#, 16#17E#, 16#17F#, 16#180#, 16#181#, 16#182#, 16#183#, 16#184#, 16#185#, 16#186#, 16#187#, 16#188#, 16#189#, 16#18A#, 16#18B#, 16#18C#, 16#18D#, 16#18E#, 16#18F#, 16#190#, 16#191#, 16#192#, 16#193#, 16#194#, 16#195#, 16#196#, 16#197#, 16#198#, 16#199#, 16#19A#, 16#19B#, 16#19C#, 16#19D#, 16#19E#, 16#19F#, 16#1A0#, 16#1A1#, 16#1A2#, 16#1A3#, 16#1A4#, 16#1A5#, 16#1A6#, 16#1A7#, 16#1A8#, 16#1A9#, 16#1AA#, 16#1AB#, 16#1AC#, 16#1AD#, 16#1AE#, 16#1AF#, 16#1B0#, 16#1B1#, 16#1B2#, 16#1B3#, 16#1B4#, 16#1B5#, 16#1B6#, 16#1B7#, 16#1B8#, 16#1B9#, 16#1BA#, 16#1BB#, 16#1BC#, 16#1BD#, 16#1BE#, 16#1BF#, 16#1C0#, 16#1C1#, 16#1C2#, 16#1C3#, 16#1C4#, 16#1C5#, 16#1C6#, 16#1C7#, 16#1C8#, 16#1C9#, 16#1CA#, 16#1CB#, 16#1CC#, 16#1CD#, 16#1CE#, 16#1CF#, 16#1D0#, 16#1D1#, 16#1D2#, 16#1D3#, 16#1D4#, 16#1D5#, 16#1D6#, 16#1D7#, 16#1D8#, 16#1D9#, 16#1DA#, 16#1DB#, 16#1DC#, 16#1DD#, 16#1DE#, 16#1DF#, 16#1E0#, 16#1E1#, 16#1E2#, 16#1E3#, 16#1E4#, 16#1E5#, 16#1E6#, 16#1E7#, 16#1E8#, 16#1E9#, 16#1EA#, 16#1EB#, 16#1EC#, 16#1ED#, 16#1EE#, 16#1EF#, 16#1F0#, 16#1F1#, 16#1F2#, 16#1F3#, 16#1F4#, 16#1F5#, 16#1F6#, 16#1F7#, 16#1F8#, 16#1F9#, 16#1FA#, 16#1FB#, 16#1FC#, 16#1FD#, 16#1FE#, 16#1FF#),
		output_size => (1, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9),
		table_size	=> 257
	);

	constant HYBRID_CODE_TABLES_ARR_C : hybrid_code_tables_arr_t(0 to 15) := (
		0  => HYBRID_CODE_TABLE0_C,
		1  => HYBRID_CODE_TABLE1_C,
		2  => HYBRID_CODE_TABLE2_C,
		3  => HYBRID_CODE_TABLE3_C,
		4  => HYBRID_CODE_TABLE4_C,
		5  => HYBRID_CODE_TABLE5_C,
		6  => HYBRID_CODE_TABLE6_C,
		7  => HYBRID_CODE_TABLE7_C,
		8  => HYBRID_CODE_TABLE8_C,
		9  => HYBRID_CODE_TABLE9_C,
		10 => HYBRID_CODE_TABLE10_C,
		11 => HYBRID_CODE_TABLE11_C,
		12 => HYBRID_CODE_TABLE12_C,
		13 => HYBRID_CODE_TABLE13_C,
		14 => HYBRID_CODE_TABLE14_C,
		15 => HYBRID_CODE_TABLE15_C
	);

end package hybrid_code_tables;